/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.swing.ImageIcon;
import javax.swing.JOptionPane;
import javax.wsdl.Binding;
import javax.wsdl.BindingOperation;
import javax.wsdl.Definition;
import javax.wsdl.Port;
import javax.wsdl.Service;
import javax.wsdl.extensions.soap.SOAPAddress;
import javax.xml.namespace.QName;

import org.apache.log4j.Logger;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.config.EndpointConfig;
import com.eviware.soapui.config.EndpointsConfig;
import com.eviware.soapui.config.InterfaceConfig;
import com.eviware.soapui.config.OperationConfig;
import com.eviware.soapui.impl.wsdl.actions.iface.InterfaceEndpointsAction;
import com.eviware.soapui.impl.wsdl.actions.iface.RemoveInterfaceAction;
import com.eviware.soapui.impl.wsdl.actions.iface.UpdateInterfaceAction;
import com.eviware.soapui.impl.wsdl.panels.iface.WsdlInterfacePanelBuilder;
import com.eviware.soapui.impl.wsdl.support.SoapRequestBuilder;
import com.eviware.soapui.impl.wsdl.support.WsdlContext;
import com.eviware.soapui.impl.wsdl.support.WsdlUtils;
import com.eviware.soapui.model.PanelBuilder;
import com.eviware.soapui.model.iface.Interface;
import com.eviware.soapui.model.iface.InterfaceListener;
import com.eviware.soapui.model.iface.Operation;
import com.eviware.soapui.model.iface.RequestBuilder;
import com.eviware.soapui.model.project.Project;
import com.eviware.soapui.model.tree.SoapUITreeNode;
import com.eviware.soapui.model.tree.nodes.InterfaceTreeNode;
import com.eviware.soapui.support.UISupport;

/**
 * WSDL implementation of Interface, maps to a WSDL Binding
 * 
 * @author Ole.Matzura
 */

public class WsdlInterface extends AbstractWsdlModelItem implements Interface
{
	private final static Logger log = Logger.getLogger( WsdlInterface.class );
	
   private InterfaceConfig interfaceConfig;
   private List<WsdlOperation> operations = new ArrayList<WsdlOperation>();
   private final WsdlProject project;
   private SoapRequestBuilder soapRequestBuilder;
   private WsdlInterfacePanelBuilder wsdlInterfacePanelBuilder;
   private WsdlContext wsdlContext;
   private ImageIcon interfaceIcon;
   private Set<InterfaceListener> listeners = new HashSet<InterfaceListener>();
   
   public WsdlInterface( WsdlProject project, InterfaceConfig interfaceConfig )
   {
      this.project = project;
      this.interfaceConfig = interfaceConfig;
      
      if( interfaceConfig.getEndpoints() == null )
         interfaceConfig.addNewEndpoints();
      
      OperationConfig [] operationConfigs = interfaceConfig.getOperationArray();
      for (int i = 0; i < operationConfigs.length; i++)
      {
         operations.add( new WsdlOperation( this, operationConfigs[i] ));
      }
      
      addAction( new InterfaceEndpointsAction( this ) );
      addAction( new UpdateInterfaceAction( this ) );
      addAction( new RemoveInterfaceAction( this ) );
      
      wsdlInterfacePanelBuilder = new WsdlInterfacePanelBuilder( this );
      interfaceIcon = SoapUI.createImageIcon("/interface.gif");
   }
   
   public PanelBuilder getPanelBuilder()
   {
      return wsdlInterfacePanelBuilder;
   }
   
   public String[] getEndpoints()
   {
      EndpointsConfig endpoints = interfaceConfig.getEndpoints();
      EndpointConfig[] endpointArray = endpoints.getEndpointArray();
      
      String [] result = new String[endpointArray.length];
      
      for( int c = 0; c < endpointArray.length; c++ )
      {
      	EndpointConfig endpoint = endpointArray[c];
      	if( endpoint.isSetLabel() )
      		result[c] = endpoint.getLabel();
      	else
      		result[c] = endpoint.getStringValue();
      }
      
		return result; 
   }

   public Operation getOperationAt(int index)
   {
      return operations.get(index);
   }

   public int getOperationCount()
   {
      return operations.size();
   }

   public String getName()
   {
      try
      {
         return interfaceConfig.getName();
      }
      catch( Exception e )
      {
         return null;
      }
   }
   
   public void setName( String name )
   {
      String oldName = getName();
      interfaceConfig.setName( name );
      notifyPropertyChanged( NAME_PROPERTY, oldName, name );
   }

   public InterfaceConfig getInterfaceConfig()
   {
      return interfaceConfig;
   }

   public WsdlOperation addNewOperation()
   {
      WsdlOperation operationImpl = new WsdlOperation( this, interfaceConfig.addNewOperation() );
      operations.add( operationImpl );
      return operationImpl;
   }

   public Project getProject()
   {
      return project;
   }

   public void addEndpoint(String endpoint)
   {
      if( endpoint == null || endpoint.trim().length() == 0 ) return;
      
      String[] endpoints = getEndpoints();
      
      // dont add the same endpoint twice
      if( Arrays.asList( endpoints ).contains( endpoint )) return;
      
		interfaceConfig.getEndpoints().addNewEndpoint().setStringValue(endpoint);
		
		notifyPropertyChanged(ENDPOINT_PROPERTY, null, endpoint);
   }
   
   public void changeEndpoint(String oldEndpoint, String newEndpoint)
	{
   	EndpointsConfig endpoints = interfaceConfig.getEndpoints();
      EndpointConfig[] endpointArray = endpoints.getEndpointArray();
      
      String [] result = new String[endpointArray.length];
      
      for( int c = 0; c < endpointArray.length; c++ )
      {
      	EndpointConfig endpoint = endpointArray[c];
      	if( endpoint.getStringValue().equals( oldEndpoint ) )
      	{
      		endpoint.setStringValue( newEndpoint );
         	notifyPropertyChanged(ENDPOINT_PROPERTY, oldEndpoint, newEndpoint);
         	break;
      	}
      }
	}

	public void removeEndpoint(String endpoint)
	{
		EndpointsConfig endpoints = interfaceConfig.getEndpoints();
      EndpointConfig[] endpointArray = endpoints.getEndpointArray();
      
      String [] result = new String[endpointArray.length];
      
      for( int c = 0; c < endpointArray.length; c++ )
      {
      	EndpointConfig ep = endpointArray[c];
      	if( ep.getStringValue().equals( endpoint ) )
      	{
      		endpoints.removeEndpoint( c );
         	notifyPropertyChanged(ENDPOINT_PROPERTY, endpoint, null );
         	break;
      	}
      }
	}

	public void setDefinition( String wsdlUrl )
   {
      String old = getDefinition();
      if( old != null && wsdlUrl.equals(old)) return;
      
      interfaceConfig.setDefinition( wsdlUrl );
      if( wsdlContext != null )
      	wsdlContext.setDefinition( wsdlUrl );
      
      notifyPropertyChanged( DEFINITION_PROPERTY, old, wsdlUrl );
   }
   
   public String getDefinition()
   {
      try
      {
         return interfaceConfig.isSetDefinition() ? interfaceConfig.getDefinition() : null;
      }
      catch( Exception e )
      {
         return null;
      }
   }
   
   public WsdlContext getWsdlContext()
   {
      if( wsdlContext == null )
         wsdlContext = new WsdlContext( getDefinition() );
      
      return wsdlContext;
   }
   
   /**
    * Used by importer so we dont need to reload the context after importing..
    * @param wsdlContext
    */
   
   public void setWsdlContext( WsdlContext wsdlContext )
   {
   	this.wsdlContext = wsdlContext;
   }

   public RequestBuilder getRequestBuilder()
   {
      if( soapRequestBuilder == null )
      {
         try
         {
            soapRequestBuilder = new SoapRequestBuilder( this );
         }
         catch (Exception e)
         {
            e.printStackTrace();
         }
      }
      return soapRequestBuilder;
   }

   public void setSoapRequestBuilder(SoapRequestBuilder builder)
   {
      this.soapRequestBuilder = builder;
   }

   public QName getBindingName()
   {
      return interfaceConfig.getBindingName() == null ? null : QName.valueOf(interfaceConfig.getBindingName());
   }

   public void setBindingName(QName name)
   {
      interfaceConfig.setBindingName( name.toString() );
   }

   public ImageIcon getIcon()
   {
      return interfaceIcon;
   }

   protected SoapUITreeNode createTreeNode()
   {
      return new InterfaceTreeNode( this );
   }

	public void updateDefinition(String url) throws Exception
	{
		WsdlContext newContext = new WsdlContext( url );
		
		Definition definition = newContext.getDefinition();
		Service service = null;
		Port port = null;
		Binding binding = null;
		
		// start by finding the old binding in the new definition
		Map serviceMap = definition.getServices();
		Iterator<String> i = serviceMap.keySet().iterator();
		while( i.hasNext() )
		{
			service = (Service) serviceMap.get( i.next() );
			Map portMap = service.getPorts();
			
			Iterator i2 = portMap.keySet().iterator();
			while( i2.hasNext() )
			{
				port = (Port) portMap.get( i2.next() );
				if( port.getBinding().getQName().equals( getBindingName() ))
				{
					binding = port.getBinding();
				}
			}
			
			if( binding != null ) break;
			service = null;
		}
		
		if( service == null && binding == null )
		{
			binding = definition.getBinding( getBindingName() );
		}

		// missing matching binding, prompt for new one to use instead (will happen if binding has been renamed)
		if( binding == null )
		{
			Map bindings = definition.getBindings();
			
			Object retval = JOptionPane.showInputDialog( SoapUI.getInstance().getFrame(), 
					"Missing matching binding [" + getBindingName() + "] in definition, select new\nbinding to map to", 
					"Map Binding", JOptionPane.QUESTION_MESSAGE, null, bindings.keySet().toArray(), null );
			
			if( retval == null )
				return;
			
			binding = (Binding) bindings.get( retval );
			setBindingName( binding.getQName() );
		}
		
		// prepare for transfer of operations/requests
		List<BindingOperation> newOperations = new ArrayList<BindingOperation>( binding.getBindingOperations() );
		Map<String,WsdlOperation> oldOperations = new HashMap<String,WsdlOperation>();
		for( int c = 0; c < operations.size(); c++ )
			oldOperations.put( operations.get( c ).getBindingOperationName(), operations.get( c ) );
		
		// clear existing from both collections
		for( int c = 0; c < newOperations.size(); c++ )
		{
			BindingOperation newOperation = newOperations.get( c );
			String bindingOperationName = newOperation.getName();
			if( oldOperations.containsKey( bindingOperationName) )
			{
				log.info( "Synchronizing existing operation [" + bindingOperationName + "]" );
				WsdlOperation wsdlOperation = oldOperations.get( bindingOperationName );
				wsdlOperation.setAction( WsdlUtils.getSoapAction( newOperation));
				wsdlOperation.setName( newOperation.getOperation().getName() );
				
				oldOperations.remove( bindingOperationName );
				newOperations.remove( c );
				c--;
			}
		}
		
		//	remove leftover operations
		i = oldOperations.keySet().iterator();
		while( i.hasNext())
		{
			String name = i.next();
			
			if( newOperations.size() > 0 )
		   {
				List<String> list = new ArrayList<String>();
				list.add( "none - delete operation" );
				for( int c = 0; c < newOperations.size(); c++ )
					list.add( newOperations.get( c ).getName() );
				
				String retval = (String) JOptionPane.showInputDialog( SoapUI.getInstance().getFrame(), 
						"Binding operation [" + name + "] not found in new interface, select new\nbinding operation to map to", 
						"Map Operation", JOptionPane.QUESTION_MESSAGE, null, list.toArray(), "none - delete operation" );
				
				if( retval == null )
				{
				   UISupport.showErrorMessage( "Aborting update of interface" );
				   return;
				}
				
				int ix = list.indexOf( retval)-1;
				
				// delete operation?
				if( ix < 0 )
				{
					deleteOperation( name );
				}
				// change operation?
				else
				{
					BindingOperation newOperation = newOperations.get( ix );
					WsdlOperation wsdlOperation = oldOperations.get( name );
					wsdlOperation.setAction( WsdlUtils.getSoapAction( newOperation));
					wsdlOperation.setName( newOperation.getOperation().getName() );
					wsdlOperation.setBindingOperationName( newOperation.getName() );

					newOperations.remove( ix );
				}

				oldOperations.remove( name );
		   }
			else 
			{
				deleteOperation( name );
				oldOperations.remove( name );
			}
			
			i = oldOperations.keySet().iterator();
		}
		
		// add leftover new operations
		for( int c = 0; c < newOperations.size(); c++ )
		{
			BindingOperation newOperation = newOperations.get( c );
			WsdlOperation wsdlOperation = addNewOperation();
			wsdlOperation.setAction( WsdlUtils.getSoapAction( newOperation));
			wsdlOperation.setName( newOperation.getOperation().getName() );
			wsdlOperation.setBindingOperationName( newOperation.getName() );
			
			notifyOperationAdded( wsdlOperation );
		}
		
		wsdlContext = newContext;
		if( soapRequestBuilder != null )
			soapRequestBuilder.setWsdlContext( wsdlContext );
		
    	setDefinition( url );

		if( port != null )
		{
	      SOAPAddress address = (SOAPAddress) WsdlUtils.getExtensiblityElement( port.getExtensibilityElements(), SOAPAddress.class );
	      if( address != null )
	         addEndpoint( address.getLocationURI() );
		}
	}

	private void deleteOperation(String bindingOperationName)
	{
		for( int c = 0; c < operations.size(); c++ )
		{
			WsdlOperation wsdlOperation = operations.get( c );
			if( wsdlOperation.getBindingOperationName().equals( bindingOperationName ))
			{
				log.info( "deleting operation [" + bindingOperationName + "]" ); 
				notifyOperationRemoved( wsdlOperation );
				wsdlOperation.release();
				
				operations.remove( c );
				interfaceConfig.removeOperation( c );
				
				return;
			}
		}
	}
	
	public void notifyOperationAdded( WsdlOperation operation )
   {
      InterfaceListener[] a = listeners.toArray( new InterfaceListener[listeners.size()] );
      
      for (int c = 0; c < a.length; c++ )
      {
         a[c].operationAdded( operation );
      }
   }
   
   public void notifyOperationRemoved( WsdlOperation operation )
   {
   	InterfaceListener[] a = listeners.toArray( new InterfaceListener[listeners.size()] );
      
      for (int c = 0; c < a.length; c++ )
      {
         a[c].operationRemoved( operation );
      }
   }
   
   public void notifyRequestAdded( WsdlRequest request )
   {
   	InterfaceListener[] a = listeners.toArray( new InterfaceListener[listeners.size()] );
      
      for (int c = 0; c < a.length; c++ )
      {
         a[c].requestAdded( request );
      }
   }
   
   public void notifyRequestRemoved( WsdlRequest request )
   {
   	InterfaceListener[] a = listeners.toArray( new InterfaceListener[listeners.size()] );
      
      for (int c = 0; c < a.length; c++ )
      {
         a[c].requestRemoved( request );
      }
   }

	public void addInterfaceListener(InterfaceListener listener)
	{
		listeners.add( listener );
	}

	public void removeInterfaceListener(InterfaceListener listener)
	{
		listeners.remove( listener );
	}

	/**
	 * Gets the endpoint url for the specified endpoint (which may be a label)
	 * 
	 * @param endpoint the endpoint to get for
	 * @return the endpoints url
	 */
	
	public String getEndpointURL(String endpoint)
	{
		EndpointConfig[] endpointArray = interfaceConfig.getEndpoints().getEndpointArray();
		for( int c = 0; c < endpointArray.length; c++ )
		{
			EndpointConfig endpointConfig = endpointArray[c];
			if( endpointConfig.isSetLabel() && endpointConfig.getLabel().equals( endpoint ))
				return endpointConfig.getStringValue();
		}
		
		return endpoint;
	}

	public Operation getOperationByName(String name)
	{
		for( WsdlOperation operation : operations )
		{
			if( operation.getName().equals( name ))
				return operation;
		}
		
		return null;
	}
}
