/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.swing.ImageIcon;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.xmlbeans.XmlException;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.config.InterfaceConfig;
import com.eviware.soapui.config.ProjectConfig;
import com.eviware.soapui.config.SoapuiProjectDocumentConfig;
import com.eviware.soapui.config.TestSuiteConfig;
import com.eviware.soapui.impl.wsdl.actions.project.AddInterfaceActionFromFile;
import com.eviware.soapui.impl.wsdl.actions.project.AddInterfaceActionFromURL;
import com.eviware.soapui.impl.wsdl.actions.project.AddNewTestSuiteAction;
import com.eviware.soapui.impl.wsdl.actions.project.RemoveProjectAction;
import com.eviware.soapui.impl.wsdl.actions.project.RenameProjectAction;
import com.eviware.soapui.impl.wsdl.support.WsdlImporter;
import com.eviware.soapui.model.iface.Interface;
import com.eviware.soapui.model.project.Project;
import com.eviware.soapui.model.project.ProjectListener;
import com.eviware.soapui.model.testsuite.TestSuite;
import com.eviware.soapui.model.tree.SoapUITreeNode;
import com.eviware.soapui.model.tree.nodes.ProjectTreeNode;

/**
 * WSDL project implementation
 * 
 * @author Ole.Matzura
 */

public class WsdlProject extends AbstractWsdlModelItem implements Project
{
   private ProjectConfig projectConfig;
   private String path;
   private List<WsdlInterface> interfaces = new ArrayList<WsdlInterface>();
   private List<WsdlTestSuite> testSuites = new ArrayList<WsdlTestSuite>();
   private Set<ProjectListener> listeners = new HashSet<ProjectListener>();
   private SoapuiProjectDocumentConfig projectDocument;
   
   private final static Log log = LogFactory.getLog( WsdlProject.class );
   private ImageIcon projectIcon;

   public WsdlProject() throws XmlException, IOException 
   {
   	projectDocument = SoapuiProjectDocumentConfig.Factory.newInstance();
      projectConfig = projectDocument.addNewSoapuiProject();
      
      init();
   }
   
   public WsdlProject(String path) throws XmlException, IOException
   {
      this.path = path;
      File file = new File(path);
      
      if( file.exists())
      {
         projectDocument = SoapuiProjectDocumentConfig.Factory.parse( file );
         projectConfig = projectDocument.getSoapuiProject();
         log.info( "Loaded project from [" + path + "]" );
         
         InterfaceConfig [] interfaceConfigs = projectConfig.getInterfaceArray();
         for (int i = 0; i < interfaceConfigs.length; i++)
         {
            interfaces.add( new WsdlInterface( this, interfaceConfigs[i] ));
         }

         TestSuiteConfig [] testSuiteConfigs = projectConfig.getTestSuiteArray();
         for (int i = 0; i < testSuiteConfigs.length; i++)
         {
            testSuites.add( new WsdlTestSuite( this, testSuiteConfigs[i] ));
         }
      }
      else 
      {
         projectDocument = SoapuiProjectDocumentConfig.Factory.newInstance();
         projectConfig = projectDocument.addNewSoapuiProject();
      }
      
      init();
   }

	private void init()
	{
		
      addAction( new AddInterfaceActionFromURL( this ) );
      addAction( new AddInterfaceActionFromFile( this ) );
      addAction( new AddNewTestSuiteAction( this ) );
      addAction( new RenameProjectAction( this ) );
      addAction( new RemoveProjectAction( this ) );
      
      projectIcon = SoapUI.createImageIcon("/project.gif");
	}

   public Interface getInterfaceAt(int index)
   {
      return interfaces.get( index );
   }

   public int getInterfaceCount()
   {
      return interfaces.size();
   }

   public String getName()
   {
      return projectConfig.getName();
   }
   
   public String getPath()
	{
		return path;
	}

	public String save() throws IOException 
   {
      if( path == null )
      {
         JFileChooser chooser = new JFileChooser();
         chooser.setDialogTitle( "Save project " + getName() );
         if( chooser.showSaveDialog( SoapUI.getInstance().getFrame() ) != JFileChooser.APPROVE_OPTION ) return null;
         
         path = chooser.getSelectedFile().getAbsolutePath();
      }
      
      File projectFile = new File( path );
		projectDocument.save( projectFile);
		log.info( "Saved project to [" + projectFile.getAbsolutePath() + "]" );
		return path;
   }

   public void setName(String name)
   {
      String old = getName();
      projectConfig.setName( name );
      notifyPropertyChanged(NAME_PROPERTY, old, name);
   }
   
   public WsdlInterface [] importWsdl( String url )
   {
      try
      {
         WsdlInterface[] result = WsdlImporter.getInstance().importWsdl( this, url );
         boolean createRequests = false;
         
         if( SoapUI.getInstance() != null )
         	createRequests = JOptionPane.showConfirmDialog( SoapUI.getInstance().getFrame(), 
         			"Create default requests for all operations", "Import WSDL", JOptionPane.YES_NO_OPTION ) == JOptionPane.YES_OPTION;
         
			if( createRequests  )
         {
         	for (WsdlInterface iface : result)
				{
         		for( int c = 0; c < iface.getOperationCount(); c++ )
         		{
         			WsdlOperation operation = (WsdlOperation) iface.getOperationAt( c );
         			WsdlRequest request = operation.addNewRequest( "Request 1");
                  try
                  {
                     request.setRequest( operation.createRequest( true ));
                  }
                  catch (Exception e)
                  {
                     e.printStackTrace();
                  }
         		}
				}
         }
         
			return result;
      }
      catch (Exception e)
      {
         log.error( "Error importing wsdl: " + e.getMessage() );
         e.printStackTrace();
      }
      return null;
   }
   
   public ProjectConfig getProjectConfig()
   {
      return projectConfig;
   }

   public WsdlInterface addNewInterface( String name )
   {
      WsdlInterface iface = new WsdlInterface( this, projectConfig.addNewInterface());
      iface.setName( name );
      interfaces.add( iface );
      notifyInterfaceAdded( iface );

      return iface;
   }
   
   public void addProjectListener(ProjectListener listener)
   {
      listeners.add( listener );
   }
   
   public void removeProjectListener(ProjectListener listener)
   {
      listeners.remove( listener );
   }
   
   public void notifyInterfaceAdded( WsdlInterface iface )
   {
      ProjectListener[] a = listeners.toArray( new ProjectListener[listeners.size()] );
      
      for (int c = 0; c < a.length; c++ )
      {
         a[c].interfaceAdded( iface );
      }
   }
   
   public void notifyInterfaceRemoved( WsdlInterface iface )
   {
      ProjectListener[] a = listeners.toArray( new ProjectListener[listeners.size()] );
      
      for (int c = 0; c < a.length; c++ )
      {
         a[c].interfaceRemoved( iface );
      }
   }
   
   
   public void notifyTestSuiteAdded( WsdlTestSuite testSuite )
   {
      ProjectListener[] a = listeners.toArray( new ProjectListener[listeners.size()] );
      
      for (int c = 0; c < a.length; c++ )
      {
         a[c].testSuiteAdded( testSuite );
      }
   }
   
   public void notifyTestSuiteRemoved( WsdlTestSuite testSuite )
   {
      ProjectListener[] a = listeners.toArray( new ProjectListener[listeners.size()] );
      
      for (int c = 0; c < a.length; c++ )
      {
         a[c].testSuiteRemoved( testSuite );
      }
   } 
   
   
   public void removeInterface(WsdlInterface iface )
   {
      int ix = interfaces.indexOf( iface );
      notifyInterfaceRemoved( iface );
      interfaces.remove( ix );
      projectConfig.removeInterface( ix );
   }
   
   public void removeTestSuite(WsdlTestSuite testSuite )
   {
      int ix = testSuites.indexOf( testSuite );
      notifyTestSuiteRemoved( testSuite );
      testSuites.remove( ix );
      projectConfig.removeTestSuite( ix );
   }

   public ImageIcon getIcon()
   {
      return projectIcon;
   }

   public int getTestSuiteCount()
   {
      return testSuites.size();
   }

   public TestSuite getTestSuiteAt(int index)
   {
      return testSuites.get( index );
   }

   public WsdlTestSuite addNewTestSuite(String name)
   {
      WsdlTestSuite testSuite = new WsdlTestSuite( this, projectConfig.addNewTestSuite());
      testSuite.setName( name );
      testSuites.add( testSuite );
      notifyTestSuiteAdded( testSuite );

      return testSuite;
   }

   protected SoapUITreeNode createTreeNode()
   {
      return new ProjectTreeNode( this );
   }
}
