/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.StringWriter;
import java.util.HashSet;
import java.util.Set;

import javax.swing.ImageIcon;
import javax.swing.JOptionPane;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.Document;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.config.CallConfig;
import com.eviware.soapui.config.CredentialsConfig;
import com.eviware.soapui.impl.wsdl.actions.request.AddToTestCaseAction;
import com.eviware.soapui.impl.wsdl.actions.request.CloneRequestAction;
import com.eviware.soapui.impl.wsdl.actions.request.DeleteRequestAction;
import com.eviware.soapui.impl.wsdl.actions.request.RenameRequestAction;
import com.eviware.soapui.impl.wsdl.panels.request.WsdlRequestPanelBuilder;
import com.eviware.soapui.model.PanelBuilder;
import com.eviware.soapui.model.iface.Interface;
import com.eviware.soapui.model.iface.Operation;
import com.eviware.soapui.model.iface.Request;
import com.eviware.soapui.model.iface.Submit;
import com.eviware.soapui.model.iface.SubmitListener;
import com.eviware.soapui.model.support.InterfaceListenerAdapter;
import com.eviware.soapui.model.tree.SoapUITreeNode;
import com.eviware.soapui.model.tree.nodes.RequestTreeNode;
import com.eviware.soapui.support.XmlUtils;

/**
 * Request implementation holding a SOAP request
 * 
 * @author Ole.Matzura
 */

public class WsdlRequest extends AbstractWsdlModelItem implements Request
{
	public static final String RESPONSE_PROPERTY = WsdlRequest.class.getName() + "@response";
   
   private CallConfig requestConfig;
   private final Operation operation;
   private PanelBuilder wsdlRequestPanelBuilder;
   private String response;
   private final static Log log = LogFactory.getLog( WsdlRequest.class );
   private IconManager iconManager;
   private Set<SubmitListener> listeners = new HashSet<SubmitListener>();

	private WsdlSubmit submitter;

	private InternalInterfaceListener internalInterfaceListener;

   public WsdlRequest( Operation operation, CallConfig callConfig )
   {
      this.operation = operation;
      this.requestConfig = callConfig;
      
      initActions();
      initEndpoints();
      initIcons();
      
      // ensure encoding
      if( requestConfig.getEncoding() == null || requestConfig.getEncoding().length() == 0 )
      {
         requestConfig.setEncoding( "UTF-8" );
      }
      
      addSubmitListener( iconManager );
    
      internalInterfaceListener = new InternalInterfaceListener();      
   }
   
   public void updateConfig(CallConfig request)
	{
		this.requestConfig = request;
	}
   
   protected PanelBuilder initPanelBuilder()
   {
      return new WsdlRequestPanelBuilder( this );
   }
   
   protected IconManager getIconManager()
   {
   	return iconManager;
   }

   protected void initIcons()
   {
   	iconManager = new IconManager();
   }

   protected void initEndpoints()
   {
      String[] endpoints = operation.getInterface().getEndpoints();
      if( getEndpoint() == null && endpoints.length > 0 )
      {
         setEndpoint( endpoints[0] );
      }
   }

   protected void initActions()
   {
      addAction( new CloneRequestAction( this ));
      addAction( new RenameRequestAction( this ));
      addAction( new DeleteRequestAction( this ));
      addAction( new AddToTestCaseAction( this ));
   }

   public PanelBuilder getPanelBuilder()
   {
      if( wsdlRequestPanelBuilder == null )
         wsdlRequestPanelBuilder = initPanelBuilder();
      
      return wsdlRequestPanelBuilder;
   }
   
   public String getRequestContent()
   {
     return requestConfig.getRequest();
   }

   public void setEndpoint(String endpoint)
   {
      String old = getEndpoint();
      if( old != null && old.equals( endpoint ))
      	return;
      
      requestConfig.setEndpoint( endpoint );
      notifyPropertyChanged( ENDPOINT_PROPERTY, old, endpoint);
   }

   public String getEndpoint()
   {
      return requestConfig.getEndpoint();
   }

   public String getEncoding()
   {
      return requestConfig.getEncoding();
   }

   public void setEncoding(String encoding)
   {
      String old = getEncoding();
      requestConfig.setEncoding( encoding );
      notifyPropertyChanged( ENCODING_PROPERTY, old, encoding );
   }
   
   public String getResponseContent()
   {
      return response;
   }

   public String getName()
   {
      try
      {
         return requestConfig.getName();
      }
      catch (Exception e)
      {
         return null;
      }
   }

   public void setName(String name)
   {
      String old = getName();
      requestConfig.setName( name );
      notifyPropertyChanged( NAME_PROPERTY, old, name );
   }

   public Operation getOperation()
   {
      return operation;
   }

   public void setRequest(String request)
   {
      String old = getRequestContent();
      requestConfig.setRequest( request );
      notifyPropertyChanged( REQUEST_PROPERTY, old, request );
   }
   
   protected CallConfig getConfig()
   {
      return requestConfig;
   }

   public void setResponse(String response)
   {
      String oldResponse = getResponseContent();
      
      // try to pretty-print
      try
      {
         Document dom = XmlUtils.parseXml( response.trim() );
         StringWriter writer = new StringWriter();
         XmlUtils.serializePretty( dom, writer );
         this.response = writer.toString();
      }
      catch( Exception e )
      {
         this.response = response;
      }
      
      // notify property-listeners
      notifyPropertyChanged( RESPONSE_PROPERTY, oldResponse, response );
   }

   public ImageIcon getIcon()
   {
   	return iconManager.getIcon();
   }

   public String getUsername()
   {
   	CredentialsConfig credentialsConfig = requestConfig.getCredentials();
   	if( credentialsConfig == null ) return null;
   	
   	return credentialsConfig.getUsername();
   }
   
   public String getPassword()
   {
   	CredentialsConfig credentialsConfig = requestConfig.getCredentials();
   	if( credentialsConfig == null ) return null;
   	
   	return credentialsConfig.getPassword();
   }
   
   public String getDomain()
   {
   	CredentialsConfig credentialsConfig = requestConfig.getCredentials();
   	if( credentialsConfig == null ) return null;
   	
   	return credentialsConfig.getDomain();
   }
   
   public void setUsername( String username )
   {
   	CredentialsConfig credentialsConfig = requestConfig.getCredentials();
   	if( credentialsConfig == null ) 
   		credentialsConfig = requestConfig.addNewCredentials();
   	
   	credentialsConfig.setUsername( username );
   }
   
   public void setPassword( String password )
   {
   	CredentialsConfig credentialsConfig = requestConfig.getCredentials();
   	if( credentialsConfig == null ) 
   		credentialsConfig = requestConfig.addNewCredentials();
   	
   	credentialsConfig.setPassword( password );
   }
   
   public void setDomain( String domain )
   {
   	CredentialsConfig credentialsConfig = requestConfig.getCredentials();
   	if( credentialsConfig == null ) 
   		credentialsConfig = requestConfig.addNewCredentials();
   	
   	credentialsConfig.setDomain( domain );
   }
   
   protected SoapUITreeNode createTreeNode()
   {
      return new RequestTreeNode( this );
   }

   protected class IconManager implements Runnable, SubmitListener
   {
      private int index = 0;
      private boolean stopped = true;
      private ImageIcon requestIcon;
      private ImageIcon [] requestExecIcons;

      public IconManager()
      {
         requestIcon = SoapUI.createImageIcon("/request.gif");

         requestExecIcons = new ImageIcon[] {
                     SoapUI.createImageIcon("/exec_request_1.gif"), 
                     SoapUI.createImageIcon("/exec_request_2.gif"),
                     SoapUI.createImageIcon("/exec_request_3.gif"),
                     SoapUI.createImageIcon("/exec_request_4.gif")
               };

      }
      
      public void stopRequestAnimation()
      {
         stopped = true;
      }
      
      public int getIndex()
      {
         return index;
      }

      public boolean isStopped()
      {
         return stopped;
      }
      
      public void startRequestAnimation()
      {
         stopped = false;
         Thread iconAnimationThread = new Thread( this );
         iconAnimationThread.start();
      }
      
      public ImageIcon getRequestIcon()
      {
      	return requestIcon;
      }
      
      public ImageIcon getIcon()
      {
         if( !iconManager.isStopped())
         {
            return requestExecIcons[iconManager.getIndex()];
         }
         
         return requestIcon;
      }

      public void run()
      {
         while( !stopped )
         {
            try
            {
               index = index >= requestExecIcons.length-1 ? 0 : index+1;
               notifyPropertyChanged( ICON_PROPERTY, null, getIcon() );
               Thread.sleep( 500 );
            }
            catch (InterruptedException e)
            {
               e.printStackTrace();
            }
         }
         
         notifyPropertyChanged( ICON_PROPERTY, null, getIcon() );
      }

		public boolean beforeSubmit(Submit submit) {
			if( submit.getRequest() == WsdlRequest.this )
				startRequestAnimation();
	      return true;
		}

		public void afterSubmit(Submit submit) {
			if( submit.getRequest() == WsdlRequest.this )
				stopRequestAnimation();
		}

   }

	public void addSubmitListener(SubmitListener listener) 
	{
		listeners.add( listener );
	}

	public void removeSubmitListener(SubmitListener listener) 
	{
		listeners.remove( listener );
	}

	public Submit submit() 
	{
      if( getEndpoint() == null || getEndpoint().trim().length() == 0 )
      {
      	JOptionPane.showMessageDialog( SoapUI.getInstance().getFrame(), 
      			"Missing endpoint for request [" + getName() + "]", "Submit Request", 
      			JOptionPane.ERROR_MESSAGE );
      	return null;
      }
		
		submitter = new WsdlSubmit( this, 
				(SubmitListener[]) listeners.toArray( new SubmitListener[listeners.size()]) );
		submitter.submitRequest();
		return submitter;
	}
	
	private class InternalInterfaceListener extends InterfaceListenerAdapter implements PropertyChangeListener
	{
		public InternalInterfaceListener()
		{
			operation.getInterface().addPropertyChangeListener( this );
	      operation.getInterface().addInterfaceListener( this );
		}
		
		/**
		 * Listen here so we can remove ourself
		 */
		
		public void requestRemoved(Request request)
		{
			if( request == WsdlRequest.this )
			{
				getOperation().getInterface().removeInterfaceListener( this );
				getOperation().getInterface().removePropertyChangeListener( this );
			}
		}
		
		public void propertyChange(PropertyChangeEvent evt)
		{
			if( evt.getPropertyName().equals( Interface.ENDPOINT_PROPERTY ))
			{
				String endpoint = getEndpoint();
				if( evt.getOldValue() != null && evt.getOldValue().equals( endpoint ))
				{
					setEndpoint( (String) evt.getNewValue() );
				}
			}
		}
	}
}
