/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.swing.ImageIcon;

import org.apache.log4j.Logger;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.config.TestCaseConfig;
import com.eviware.soapui.config.TestStepConfig;
import com.eviware.soapui.impl.wsdl.actions.testcase.RemoveTestCaseAction;
import com.eviware.soapui.impl.wsdl.actions.testcase.RenameTestCaseAction;
import com.eviware.soapui.impl.wsdl.panels.testcase.WsdlTestCasePanelBuilder;
import com.eviware.soapui.impl.wsdl.teststeps.TransferResponseValuesTestStep;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlDelayTestStep;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlGotoTestStep;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestRequestStep;
import com.eviware.soapui.model.PanelBuilder;
import com.eviware.soapui.model.testsuite.TestCase;
import com.eviware.soapui.model.testsuite.TestRunListener;
import com.eviware.soapui.model.testsuite.TestRunner;
import com.eviware.soapui.model.testsuite.TestStep;
import com.eviware.soapui.model.testsuite.TestSuite;
import com.eviware.soapui.model.tree.SoapUITreeNode;
import com.eviware.soapui.model.tree.nodes.TestCaseTreeNode;

/**
 * TestCase implementation for WSDL projects
 *  
 * @author Ole.Matzura
 */

public class WsdlTestCase extends AbstractWsdlModelItem implements TestCase
{
	private final WsdlTestSuite testSuite;
   private final TestCaseConfig config;

   private List<WsdlTestStep> testSteps = new ArrayList<WsdlTestStep>();
   private ImageIcon testCaseIcon;

   private final static Logger logger = Logger.getLogger( WsdlTestCase.class );
   private PanelBuilder panelBuilder;
   
   private Set<TestRunListener> testRunListeners = new HashSet<TestRunListener>();
   
   public WsdlTestCase(WsdlTestSuite testSuite, TestCaseConfig config)
   {
      this.testSuite = testSuite;
      this.config = config;
      
      TestStepConfig [] testStepConfigs = config.getTestStepArray();
      for (int i = 0; i < testStepConfigs.length; i++)
      {
         TestStepConfig tsc = testStepConfigs[i];
         addTestStepFromConfig(tsc);
      }
      
      testCaseIcon = SoapUI.createImageIcon("/testCase.gif");
      
      addAction( new RenameTestCaseAction( this ) );
      addAction( new RemoveTestCaseAction( this ) );
   }

	private WsdlTestStep addTestStepFromConfig(TestStepConfig tsc)
	{
		String type = tsc.getType();
		WsdlTestStep newStep = null;
		if( type.equals( "request" ))
		{
			newStep = new WsdlTestRequestStep( this, tsc );
		}
		else if( type.equals( "transfer") )
		{
			newStep = new TransferResponseValuesTestStep( this, tsc );
		}
		else if( type.equals( "delay" ))
		{
			newStep = new WsdlDelayTestStep( this, tsc );
		}
		else if( type.equals( "goto" ))
		{
			newStep = new WsdlGotoTestStep( this, tsc );
		}
		else logger.error( "Unknown request step type: " + type );
		
		if( newStep != null )
			testSteps.add( newStep );
		
		return newStep;
	}

   public TestSuite getTestSuite()
   {
      return testSuite;
   }

   public WsdlTestStep cloneStep( WsdlTestStep testStep, String name )
   {
   	TestStepConfig testStepConfig = (TestStepConfig) config.addNewTestStep().set( testStep.getConfig().copy() );
   	WsdlTestStep newStep = addTestStepFromConfig( testStepConfig );
   	newStep.setName( name );
      ((WsdlTestSuite)getTestSuite()).notifyTestStepAdded( newStep, testSteps.size()-1  );
		return newStep;
   }
   
   public String getMessage()
   {
      return null;
   }

   public TestStatus getStatus()
   {
      return TestStatus.UNKNOWN;
   }

   public TestStep getTestStepAt(int index)
   {
      return testSteps.get( index );
   }

   public int getTestStepCount()
   {
      return testSteps.size();
   }

   public String getName()
   {
      return config.getName();
   }

   public ImageIcon getIcon()
   {
     return testCaseIcon;
   }

   public void setName(String name)
   {
      String old = getName();
      config.setName( name );
      notifyPropertyChanged(NAME_PROPERTY, old, name);
   }

   public WsdlTestRequestStep addTestRequestStep(WsdlRequest request, String name )
   {
      TestStepConfig testStep = config.addNewTestStep();
      testStep.setType( "request" );
      WsdlTestRequestStep testRequestStep = new WsdlTestRequestStep( this, testStep, request );
      testRequestStep.setName( name );
      testSteps.add( testRequestStep );
      
      ((WsdlTestSuite)getTestSuite()).notifyTestStepAdded( testRequestStep, testSteps.size()-1 );
      
      return testRequestStep;
   }

   public WsdlDelayTestStep addDelayTestStep( int ix )
   {
      TestStepConfig testStepConfig = ix == -1 ? config.addNewTestStep() : config.insertNewTestStep( ix );
      testStepConfig.setType( "delay" );
      WsdlDelayTestStep waitStep = new WsdlDelayTestStep( this, testStepConfig );
      
      if( ix == -1 )
      	testSteps.add( waitStep );
      else
      	testSteps.add( ix, waitStep );
      
      ((WsdlTestSuite)getTestSuite()).notifyTestStepAdded( waitStep, ix == -1 ? testSteps.size()-1 : ix  );
      
      return waitStep;
   }

   public WsdlGotoTestStep addGotoTestStep( int ix )
   {
      TestStepConfig testStepConfig = ix == -1 ? config.addNewTestStep() : config.insertNewTestStep( ix );
      testStepConfig.setType( "goto" );
      WsdlGotoTestStep gotoStep = new WsdlGotoTestStep( this, testStepConfig );
      gotoStep.setName( "Goto" );
      
      if( ix == -1 )
      	testSteps.add( gotoStep );
      else
      	testSteps.add( ix, gotoStep );
      
      ((WsdlTestSuite)getTestSuite()).notifyTestStepAdded( gotoStep, ix == -1 ? testSteps.size()-1 : ix  );
      
      return gotoStep;
   }

   
   public TransferResponseValuesTestStep addTransferResponseValuesStep(int ix)
   {
      TestStepConfig testStepConfig = ix == -1 ? config.addNewTestStep() : config.insertNewTestStep( ix );
      testStepConfig.setType( "transfer" );
      TransferResponseValuesTestStep testStep = new TransferResponseValuesTestStep( this, testStepConfig );
      testStep.setName( "Transfer values" );
      
      if( ix == -1 )
      	testSteps.add( testStep );
      else
      	testSteps.add( ix, testStep );
      
      ((WsdlTestSuite)getTestSuite()).notifyTestStepAdded( testStep, ix == -1 ? testSteps.size()-1 : ix  );
      
      return testStep;
   }
   
   
   public void removeTestStep(WsdlTestStep testStep)
   {
      int ix = testSteps.indexOf( testStep );
      if( ix == -1 )
      {
         logger.error( "TestStep [" + testStep.getName() + "] passed to removeTestStep in testCase [" + 
               getName() + "] not found" );
         return;
      }
      
      ((WsdlTestSuite)getTestSuite()).notifyTestStepRemoved( testStep, ix );
      testSteps.remove( ix );
      config.removeTestStep( ix );
   }
   
   protected SoapUITreeNode createTreeNode()
   {
      return new TestCaseTreeNode( this );
   }

   public PanelBuilder getPanelBuilder()
   {
      if( panelBuilder == null )
         panelBuilder = new WsdlTestCasePanelBuilder( this );
      
      return panelBuilder;
   }

   public TestRunner run() 
   {
   	TestRunner runner = new WsdlTestRunner( this );
   	runner.start();
   	return runner;
	}

	public void addTestRunListener(TestRunListener listener) {
		testRunListeners.add( listener );
	}

	public void removeTestRunListener(TestRunListener listener) {
		testRunListeners.remove( listener );
	}
	
	TestRunListener [] getTestRunListeners()
	{
		return testRunListeners.toArray( new TestRunListener[testRunListeners.size()]);
	}

	WsdlTestStep[] getTestSteps() 
	{
		return testSteps.toArray( new WsdlTestStep[testSteps.size()] );
 	}

	public int getIndexOfTestStep(TestStep step)
	{
		return testSteps.indexOf( step );
	}

	/**
	 * Moves a step by the specified offset
	 * 
	 * @param ix
	 * @param offset
	 */
	
	public void moveStep(int ix, int offset)
	{
		if( offset == 0 ) return;
		WsdlTestStep step = testSteps.get( ix );

		((WsdlTestSuite)getTestSuite()).notifyTestStepRemoved(step, ix );

		testSteps.remove( ix );
		testSteps.add( ix+offset, step );

		TestStepConfig [] configs = new TestStepConfig[testSteps.size()];
		
		for( int c = 0; c < testSteps.size(); c++ )
		{
			if( offset > 0 )
			{
				if( c < ix )
					configs[c] = (TestStepConfig) config.getTestStepArray(c).copy();
				else if( c < (ix+offset))
					configs[c] = (TestStepConfig) config.getTestStepArray(c+1).copy();
				else if( c == ix+offset )
					configs[c] = (TestStepConfig) config.getTestStepArray(ix).copy();
				else
					configs[c] = (TestStepConfig) config.getTestStepArray(c).copy();
			}
			else
			{
				if( c < ix+offset )
					configs[c] = (TestStepConfig) config.getTestStepArray(c).copy();
				else if( c == ix+offset )
					configs[c] = (TestStepConfig) config.getTestStepArray(ix).copy();
				else if( c <= ix )
					configs[c] = (TestStepConfig) config.getTestStepArray(c-1).copy();
				else
					configs[c] = (TestStepConfig) config.getTestStepArray(c).copy();				
			}
		}
		
		config.setTestStepArray( configs );
		for( int c = 0; c < configs.length; c++ )
		{
			testSteps.get( c ).resetConfigOnMove( config.getTestStepArray( c ));
		}
		
		((WsdlTestSuite)getTestSuite()).notifyTestStepAdded(step, ix+offset );
	}

	public int getTestStepIndexByName(String stepName)
	{
		for( int c = 0; c < testSteps.size(); c++ )
		{
			if( testSteps.get( c ).getName().equals( stepName ))
				return c;
		}
		
		return -1;
	}
	
	public TestStep findPreviousStepOfType( TestStep referenceStep, Class stepClass )
	{
		int currentStepIndex = getIndexOfTestStep( referenceStep );
		int ix = currentStepIndex - 1 ;
		while( ix >= 0 && !getTestStepAt( ix ).getClass().equals( stepClass ))
		{
			ix--;
		}
		
		return ix == -1 ? null : getTestStepAt( ix );
	}
	
	public TestStep findNextStepOfType( TestStep referenceStep, Class stepClass )
	{
		int currentStepIndex = getIndexOfTestStep( referenceStep );
		int ix = currentStepIndex + 1 ;
		while( ix < getTestStepCount() && !getTestStepAt( ix ).getClass().equals( stepClass ))
		{
			ix++;
		}
		
		return ix >= getTestStepCount() ? null : getTestStepAt( ix );
	}
}
