/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl;

import java.util.ArrayList;
import java.util.List;

import com.eviware.soapui.model.testsuite.TestCase;
import com.eviware.soapui.model.testsuite.TestRunListener;
import com.eviware.soapui.model.testsuite.TestRunner;
import com.eviware.soapui.model.testsuite.TestStep;

/**
 * WSDL Test Runner
 * 
 * @author Ole.Matzura
 */

public class WsdlTestRunner implements Runnable, TestRunner
{
	private TestRunListener [] listeners;
	private List<Long> timestamps = new ArrayList<Long>();
	private final WsdlTestCase testCase;
	private Status status;
	private WsdlTestStep currentStep;
	private Exception error;
	private int currentStepIndex;
	private Thread thread;
	private int gotoStepIndex;
	
	
	public WsdlTestRunner( WsdlTestCase testCase )
	{
		this.testCase = testCase;
		listeners = testCase.getTestRunListeners();
		status = Status.INITIALIZED;
	}
	
	public void start() 
	{
		thread = new Thread( this );
		thread.start();
		status = Status.RUNNING;
	}
	
	public void cancel()
	{
		if( status == Status.CANCELED ) return;
		currentStep.cancel();
		status = Status.CANCELED;
		notifyAfterRun();
	}
	
	public Status getStatus()
	{
		return status;
	}

	public void run() 
	{
		gotoStepIndex = -1;
		
		timestamps.add( System.currentTimeMillis() );
		for (int i = 0; i < listeners.length; i++) 
		{
			if( !listeners[i].beforeRun( this ))
				return;
		}
		
		try {
			WsdlTestStep[] testSteps = testCase.getTestSteps();

			for ( currentStepIndex = 0; status != Status.CANCELED && currentStepIndex < testSteps.length; currentStepIndex++) 
			{
				currentStep = testSteps[currentStepIndex];
				for (int i = 0; i < listeners.length; i++) 
				{
					if (!listeners[i].beforeStep(this ))
						return;
				}
				timestamps.add(System.currentTimeMillis());
				currentStep.run( this );
				timestamps.add(System.currentTimeMillis());

				if( status == Status.CANCELED )
					break;
				
				for (int i = 0; i < listeners.length; i++) 
				{
					if (!listeners[i].afterStep( this ))
						return;
				}
				
				if( gotoStepIndex != -1 )
				{
					currentStepIndex = gotoStepIndex-1;
					gotoStepIndex = -1;
				}
			}
		} 
		catch (Exception e) 
		{
			e.printStackTrace();
			status = Status.ERROR;
			error = e;
		}
		finally
		{
			if( status != Status.CANCELED )
			{
				notifyAfterRun();
			}
			timestamps.add( System.currentTimeMillis() );
		}
	}

	private void notifyAfterRun()
	{
		for (int i = 0; i < listeners.length; i++) 
		{
			listeners[i].afterRun( this );
		}
	}

	public TestStep getCurrentStep() {
		return currentStep;
	}

	public int getCurrentStepIndex() {
		return currentStepIndex;
	}

	public TestCase getTestCase() {
		return testCase;
	}

	public Thread getThread()
	{
		return thread;
	}

	public void waitUntilFinished()
	{
		synchronized( thread )
		{
			try
			{
				thread.wait();
			}
			catch (InterruptedException e)
			{
				throw new RuntimeException( e );
			}
		}
	}

	public void gotoStep(int index)
	{
		gotoStepIndex  = index;
	}
}