/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.support;

import java.io.File;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.namespace.QName;

import org.apache.log4j.Logger;
import org.apache.xmlbeans.SchemaTypeLoader;
import org.apache.xmlbeans.SimpleValue;
import org.apache.xmlbeans.XmlBeans;
import org.apache.xmlbeans.XmlCursor;
import org.apache.xmlbeans.XmlException;
import org.apache.xmlbeans.XmlObject;
import org.apache.xmlbeans.XmlOptions;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

import com.eviware.soapui.SoapUI;

/**
 * XML-Schema related tools
 * 
 * @author Ole.Matzura
 */

public class SchemaUtils
{
   private final static Logger log = Logger.getLogger( SchemaUtils.class );
   
   public static SchemaTypeLoader loadSchemaTypes(String wsdlUrl) throws SchemaException
   {
   	try
		{
			log.info("Loading schema types from [" + wsdlUrl + "]");
			ArrayList<XmlObject> schemas = new ArrayList<XmlObject>(getSchemas(
					wsdlUrl).values());

			// remove imports
			for (XmlObject s : schemas)
			{
				removeImports(s);
			}

			SchemaTypeLoader schemaTypes = loadSchemaTypes(schemas);
			return schemaTypes;
		}
		catch (XmlException e)
		{
			throw new SchemaException( "Error loading schema types", e );
		}
   }

   public static SchemaTypeLoader loadSchemaTypes(List<XmlObject> schemas) throws SchemaException
   {
      XmlOptions options = new XmlOptions();
      options.setCompileNoValidation();
      options.setCompileNoPvrRule();
      options.setCompileDownloadUrls();
      options.setCompileNoUpaRule();
      options.setValidateTreatLaxAsSkip();
      
      ArrayList errorList = new ArrayList();
      options.setErrorListener( errorList );

      try
		{
      	schemas.add( XmlObject.Factory.parse( SoapUI.class.getResource("/soapEncoding.xsd") ));
      	
			return XmlBeans.loadXsd(
					schemas.toArray(new XmlObject[schemas.size()]), options);
		}
		catch (Exception e)
		{
			throw new SchemaException( e, errorList );
		}
   }
   
   public static Map<String,XmlObject> getSchemas( String wsdlUrl ) throws SchemaException
   {
   	Map<String,XmlObject> result = new HashMap<String,XmlObject>();
   	getSchemas( wsdlUrl, result );
   	return result;
   }
   
   public static void getSchemas( String wsdlUrl, Map<String,XmlObject> result ) throws SchemaException
   {
   	if( result.containsKey( wsdlUrl ))
   		return; 
   	
   	log.info( "Getting schema " + wsdlUrl );
   	
   	ArrayList errorList = new ArrayList();
      
   	
      try
		{
			XmlOptions options = new XmlOptions();
			options.setCompileNoValidation();
			options.setSaveUseOpenFrag();
			options.setErrorListener( errorList );
			options.setSaveSyntheticDocumentElement(new QName( "http://www.w3.org/2001/XMLSchema", "schema"));

			XmlObject xmlObject = XmlObject.Factory.parse(new URL(wsdlUrl), options);

			Document dom = (Document) xmlObject.getDomNode();
			Node domNode = dom.getDocumentElement();
			if (domNode.getLocalName().equals("schema")
					&& domNode.getNamespaceURI().equals(
							"http://www.w3.org/2001/XMLSchema"))
			{
				result.put(wsdlUrl, xmlObject);
			}
			else
			{
				XmlObject[] schemas = xmlObject
						.selectPath("declare namespace s='http://www.w3.org/2001/XMLSchema' .//s:schema");

				for (int i = 0; i < schemas.length; i++)
				{
					XmlCursor xmlCursor = schemas[i].newCursor();
					String xmlText = xmlCursor.getObject().xmlText(options);
					schemas[i] = XmlObject.Factory.parse(xmlText, options);
					schemas[i].documentProperties().setSourceName(wsdlUrl);

					result.put(wsdlUrl, schemas[i]);
				}

				XmlObject[] wsdlImports = xmlObject
						.selectPath("declare namespace s='http://schemas.xmlsoap.org/wsdl/' .//s:import");
				for (int i = 0; i < wsdlImports.length; i++)
				{
					String location = wsdlImports[i].getDomNode().getAttributes()
							.getNamedItem("location").getNodeValue();
					if (location != null)
					{
						if (location.startsWith("file:")
								|| location.indexOf("://") > 0)
						{
							getSchemas(location, result);
						}
						else
						{
							getSchemas(joinRelativeUrl(wsdlUrl, location), result);
						}
					}
				}
			}

			XmlObject[] schemas = result.values().toArray(
					new XmlObject[result.size()]);

			for (int c = 0; c < schemas.length; c++)
			{
				xmlObject = schemas[c];

				XmlObject[] schemaImports = xmlObject
						.selectPath("declare namespace s='http://www.w3.org/2001/XMLSchema' .//s:import/@schemaLocation");
				for (int i = 0; i < schemaImports.length; i++)
				{
					String location = ((SimpleValue) schemaImports[i])
							.getStringValue();
					if (location != null)
					{
						if (location.startsWith("file:")
								|| location.indexOf("://") > 0)
						{
							getSchemas(location, result);
						}
						else
						{
							getSchemas(joinRelativeUrl(wsdlUrl, location), result);
						}
					}
				}

				XmlObject[] schemaIncludes = xmlObject
						.selectPath("declare namespace s='http://www.w3.org/2001/XMLSchema' .//s:include/@schemaLocation");
				for (int i = 0; i < schemaIncludes.length; i++)
				{
					String location = ((SimpleValue) schemaIncludes[i])
							.getStringValue();
					if (location != null)
					{
						if (location.startsWith("file:")
								|| location.indexOf("://") > 0)
						{
							getSchemas(location, result);
						}
						else
						{
							getSchemas(joinRelativeUrl(wsdlUrl, location), result);
						}
					}
				}
			}
		}
		catch (Exception e)
		{
			throw new SchemaException( e, errorList );
		}
   }
   
   public static Map<String,XmlObject> getDefinitionUrls( String wsdlUrl ) throws Exception
   {
   	Map<String,XmlObject> result = new HashMap<String,XmlObject>();
   	
      XmlObject xmlObject = XmlObject.Factory.parse(new URL(wsdlUrl));
      result.put( wsdlUrl, xmlObject );

    	XmlObject [] wsdlImports = xmlObject.selectPath("declare namespace s='http://schemas.xmlsoap.org/wsdl/' .//s:import");
      for (int i = 0; i < wsdlImports.length; i++)
      {
         String location = wsdlImports[i].getDomNode().getAttributes().getNamedItem( "location" ).getNodeValue();
         if( location != null )
         {
            if( location.startsWith( "file:" ) || location.indexOf( "://") > 0 )
            {
               result.putAll( getDefinitionUrls( location ));
            }
            else
            {
               result.putAll( getDefinitionUrls( joinRelativeUrl( wsdlUrl, location ) ));
            }
         }
      }
      
      XmlObject[] schemaImports = xmlObject.selectPath("declare namespace s='http://www.w3.org/2001/XMLSchema' .//s:import/@schemaLocation");
      for (int i = 0; i < schemaImports.length; i++)
      {
         String location = ((SimpleValue)schemaImports[i]).getStringValue();
         if( location != null )
         {
            if( location.startsWith( "file:" ) || location.indexOf( "://") > 0 )
            {
               result.putAll( getDefinitionUrls( location ));
            }
            else
            {
               result.putAll( getDefinitionUrls( joinRelativeUrl( wsdlUrl, location )));
            }
         }
      }
      
      XmlObject[] schemaIncludes = xmlObject.selectPath("declare namespace s='http://www.w3.org/2001/XMLSchema' .//s:include/@schemaLocation");
      for (int i = 0; i < schemaIncludes.length; i++)
      {
         String location = ((SimpleValue)schemaIncludes[i]).getStringValue();
         if( location != null  )
         {
            if( location.startsWith( "file:" ) || location.indexOf( "://") > 0 )
            {
               result.putAll( getDefinitionUrls( location ));
            }
            else
            {
               result.putAll( getDefinitionUrls( joinRelativeUrl( wsdlUrl, location )));
            }
         }
      }
	      
      return result;
   }
   
   public static String joinRelativeUrl( String root, String url )
   {
      int ix = root.startsWith("file:") ? root.lastIndexOf( File.separatorChar ) : root.lastIndexOf( '/' );
   	if( ix == -1 ) ix = root.lastIndexOf( '/' );
   	return root.substring( 0, ix+1 ) + url;
   }

	private static void removeImports(XmlObject xmlObject) throws XmlException
	{
		 XmlObject[] imports = xmlObject
       	.selectPath("declare namespace s='http://www.w3.org/2001/XMLSchema' .//s:import");
		 
		 for( int c = 0; c < imports.length; c++ )
		 {
			 imports[c].newCursor().removeXml();
		 }
		 
		 XmlObject[] includes = xmlObject
     		.selectPath("declare namespace s='http://www.w3.org/2001/XMLSchema' .//s:include");
		 
		 for( int c = 0; c < includes.length; c++ )
		 {
			 includes[c].newCursor().removeXml();
		 }
	}
}
