/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.support;

import java.util.ArrayList;

import javax.wsdl.Definition;
import javax.wsdl.factory.WSDLFactory;
import javax.wsdl.xml.WSDLReader;
import javax.xml.namespace.QName;

import org.apache.log4j.Logger;
import org.apache.xmlbeans.SchemaType;
import org.apache.xmlbeans.SchemaTypeLoader;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.support.ProgressDialog;
import com.eviware.soapui.support.SwingWorker;
import com.eviware.soapui.support.UISupport;

/**
 * Holder for WSDL4J Definitions and related SchemaTypeLoader types
 * 
 * @author Ole.Matzura
 */

public class WsdlContext
{
   private String url;
   private Definition definition;
   private SchemaTypeLoader schemaTypes;
   private boolean loaded;
	private SchemaException schemaException;
	
	 private final static Logger log = Logger.getLogger( WsdlContext.class );

   public WsdlContext( String url )
   {
      this.url = url;
   }
   
   public Definition getDefinition() throws Exception
   {
      if( !loaded )
         load();
      
      return definition;
   }
   
   public void setDefinition( String url ) 
   {
   	this.url = url;
   	loaded = false;
   }
   
   public void load() throws Exception
   {
      ProgressDialog progressDialog = SoapUI.getInstance() == null ? null : 
      	new ProgressDialog( "Progress", "Loading WSDL", 1, "Loading definition..", null );

      Loader loader = new Loader( progressDialog );
      loader.start();
      if( progressDialog != null )
      {
      	progressDialog.setVisible( true );
      }
      
      if( loader.get() != null )
      {
         Exception exception = (Exception)loader.get();
         if( exception instanceof SchemaException )
         {
         	schemaException = (SchemaException) exception;
         	ArrayList errorList = schemaException.getErrorList();
         	
         	log.error( "Error loading schema types from " + url + ", see log for details" );
         	for( int c = 0; c < errorList.size(); c++ )
         	{
         		log.error( errorList.get( c ).toString() );
         	}
         	
        		UISupport.showErrorMessage( "Error loading schema types from " + url + ", see log for details" );
         }
         else throw exception;
      }
      
      loaded = true;
   }

   public SchemaTypeLoader getSchemaTypes() throws Exception
   {
      if( !loaded )
         load();
      
      return schemaTypes;
   }
   
   public SchemaException getSchemaException()
   {
   	return schemaException;
   }

   private class Loader extends SwingWorker
   {
      private ProgressDialog progressDialog;

      public Loader( ProgressDialog progressDialog )
      {
         this.progressDialog = progressDialog;
      }
      
      public Object construct()
      {
         try
         {
            WSDLFactory factory = WSDLFactory.newInstance();
            WSDLReader wsdlReader = factory.newWSDLReader();
            wsdlReader.setFeature("javax.wsdl.verbose", true);
            wsdlReader.setFeature("javax.wsdl.importDocuments", true);
            
            if( progressDialog != null )
            	progressDialog.setProgress( 1, "Loading wsdl.." );
            definition = wsdlReader.readWSDL(url);
         }
         catch (Exception e)
         {
            return e;
         }
         
         try
         {
            if( progressDialog != null )
            	progressDialog.setProgress( 1, "Loading schema types" );
            
            schemaTypes = SchemaUtils.loadSchemaTypes(url);
            return null;
         }
         catch (SchemaException e)
         {
            return e;
         }
      }

      public void finished()
      {
      	if( progressDialog != null )
      		progressDialog.setVisible( false );
      }
   }

   public boolean hasSchemaTypes() throws Exception
   {
      if( !loaded )
         load();
      
      return schemaTypes != null;
   }

   public SchemaType findType(QName typeName) throws Exception
   {
      if( !loaded )
         load();
      
      return schemaTypes == null ? null : schemaTypes.findType( typeName );
   }

   public String getUrl()
   {
      return url;
   }
}
