/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.support;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.wsdl.Binding;
import javax.wsdl.Port;
import javax.wsdl.Service;
import javax.wsdl.extensions.soap.SOAPAddress;
import javax.wsdl.factory.WSDLFactory;
import javax.wsdl.xml.WSDLReader;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.wsdl.WsdlInterface;
import com.eviware.soapui.impl.wsdl.WsdlProject;
import com.eviware.soapui.model.iface.Interface;

/**
 * Importer for WsdlInterfaces from WSDL urls / files
 * 
 * @author Ole.Matzura
 */

public class WsdlImporter
{
   private final SoapUI soapUI;
   private WSDLFactory factory;
   private WSDLReader wsdlReader;
   private List<BindingImporter> bindingImporters = new ArrayList<BindingImporter>();
   private static WsdlImporter instance;
   
   private final static Log log = LogFactory.getLog( WsdlImporter.class );
   
   private WsdlImporter(SoapUI soapUI)
   {
      this.soapUI = soapUI;

      try
      {
         factory = WSDLFactory.newInstance();
         wsdlReader = factory.newWSDLReader();
         wsdlReader.setFeature("javax.wsdl.verbose", true);
         wsdlReader.setFeature("javax.wsdl.importDocuments", true);
         
         bindingImporters.add( new SoapHttpBindingImporter() );
      }
      catch (Exception e)
      {
         e.printStackTrace();
      }
   }

   public static WsdlImporter getInstance()
   {
      if( instance == null )
         instance = new WsdlImporter( SoapUI.getInstance() );
      
      return instance;
   }
   
   public WsdlInterface [] importWsdl( WsdlProject project, String wsdlUrl ) throws Exception
   {
      WsdlContext wsdlContext = new WsdlContext( wsdlUrl );
      wsdlContext.load();
      
      List<Interface> result = new ArrayList<Interface>();
      
      Map bindingMap = wsdlContext.getDefinition().getBindings();
      if( bindingMap.isEmpty() )
      {
         Map serviceMap = wsdlContext.getDefinition().getServices();
         if( serviceMap.isEmpty() )
            throw new RuntimeException( "Missing bindings and services in [" + wsdlUrl + "]" );
         
         Iterator i = serviceMap.values().iterator();
         while( i.hasNext() )
         {
            Service service = (Service) i.next();
            Map portMap = service.getPorts();
            Iterator i2 = portMap.values().iterator();
            while( i2.hasNext() )
            {
               Port port = (Port) i2.next();
               
               Binding binding = port.getBinding();
               WsdlInterface iface = importBinding(project, wsdlContext, binding);
               if( iface != null )
               {
	               SOAPAddress address = (SOAPAddress) WsdlUtils.getExtensiblityElement( port.getExtensibilityElements(), SOAPAddress.class );
	               if( address != null )
	                  iface.addEndpoint( address.getLocationURI() );
	               
	               result.add( iface );
	               iface.setWsdlContext( wsdlContext );
               }
            }
         }
      }
      else
      {
         Iterator i = bindingMap.values().iterator();
         while( i.hasNext() )
         {
            Binding binding = (Binding) i.next();
            WsdlInterface iface = importBinding(project, wsdlContext, binding);
            if( iface != null )
            {
            	result.add( iface );
            	iface.setWsdlContext( wsdlContext );
            }
         }
      }
      
      return result.toArray( new WsdlInterface[result.size()]);
   }

   private WsdlInterface importBinding(WsdlProject project, WsdlContext wsdlContext, Binding binding ) throws Exception
   {
      log.info( "Finding importer for " + binding.getQName() );
      for( int c = 0; c < bindingImporters.size(); c++ )
      {
         BindingImporter importer = bindingImporters.get( c );
         if( importer.canImport( binding ))
         {
            log.info( "Importing binding " + binding.getQName() );
            WsdlInterface iface = importer.importBinding( project, wsdlContext, binding );
            iface.setDefinition( wsdlContext.getUrl() );
            return iface;
         }
      }
      log.info( "Missing importer for " + binding.getQName() );
      
      return null;
   }
   
  
  
}
