/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.teststeps;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.swing.ImageIcon;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.log4j.Logger;
import org.apache.xmlbeans.XmlObject;
import org.apache.xmlbeans.XmlString;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.config.TestStepConfig;
import com.eviware.soapui.config.TransferValuesStepConfig;
import com.eviware.soapui.config.ValueTransferConfig;
import com.eviware.soapui.config.ValueTransferSourceTypeConfig;
import com.eviware.soapui.config.ValueTransferTargetTypeConfig;
import com.eviware.soapui.impl.wsdl.WsdlTestCase;
import com.eviware.soapui.impl.wsdl.WsdlTestStep;
import com.eviware.soapui.impl.wsdl.panels.testcase.TransferResponseValuesTestStepPanelBuilder;
import com.eviware.soapui.model.PanelBuilder;
import com.eviware.soapui.model.testsuite.TestRunner;
import com.eviware.soapui.support.XmlUtils;

/**
 * WsdlTestStep for transferring values from a WsdlTestRequest response to a 
 * WsdlTestRequest request using XPath expressions
 * 
 * @author Ole.Matzura
 */

public class TransferResponseValuesTestStep extends WsdlTestStep
{
	private TransferValuesStepConfig transferStepConfig;
	private TransferResponseValuesTestStepPanelBuilder panelBuilder;
	private final static Logger log = Logger.getLogger( TransferResponseValuesTestStep.class );
	private boolean canceled;
	private List<ValueTransfer> transfers = new ArrayList<ValueTransfer>();
	private Set<ValueTransfer> lastTransfers = new HashSet<ValueTransfer>();
	private ImageIcon transferIcon;

	public TransferResponseValuesTestStep(WsdlTestCase testCase, TestStepConfig config)
	{
		super(testCase, config);
	
      if( config.getConfig() != null )
      {
			transferStepConfig = (TransferValuesStepConfig) config.getConfig().changeType(TransferValuesStepConfig.type);
			for( int c = 0; c < transferStepConfig.sizeOfTransfersArray(); c++ )
			{
				transfers.add( new ValueTransfer( transferStepConfig.getTransfersArray( c )));
			}				
      }
      else
      {
         transferStepConfig = (TransferValuesStepConfig) config.addNewConfig().changeType( TransferValuesStepConfig.type );
      }
      
      transferIcon = SoapUI.createImageIcon("/value_transfer.gif");
	}
	
	public TransferValuesStepConfig getTransferConfig()
	{
		return transferStepConfig;
	}
	
	protected void resetConfigOnMove(TestStepConfig config)
	{
		super.resetConfigOnMove(config);
		
		transferStepConfig = (TransferValuesStepConfig) config.getConfig().changeType(TransferValuesStepConfig.type);
		for( int c = 0; c < transferStepConfig.sizeOfTransfersArray(); c++ )
		{
			transfers.get( c ).setConfig( transferStepConfig.getTransfersArray( c ));
		}		
	}

	public ImageIcon getIcon()
	{
		return transferIcon;
	}

	public void run( TestRunner runner )
	{
		canceled = false;
		lastTransfers.clear();
		
		WsdlTestRequestStep previousStep = (WsdlTestRequestStep) getTestCase().findPreviousStepOfType( 
				this, WsdlTestRequestStep.class );
		if( previousStep == null )
		{
			log.error( "Failed to find previous request step from [" + getName() + "]" );
			return;
		}
		
		WsdlTestRequestStep nextStep = (WsdlTestRequestStep) getTestCase().findNextStepOfType( 
				this, WsdlTestRequestStep.class );
		if( nextStep == null )
		{
			log.error( "Failed to find next request step from [" + getName() + "]" );
			return;
		}
		
		runTransfer( previousStep.getTestRequest(), nextStep.getTestRequest());
	}
	
	public void runTransfer( WsdlTestRequest sourceRequest, WsdlTestRequest targetRequest )
	{
		for( int c = 0; c < transfers.size(); c++ )
		{
			try
			{
				if( canceled ) 
					return;
				
				transferValues( sourceRequest, targetRequest, transfers.get( c ));
			}
			catch (Exception e)
			{
				log.error( e.getMessage() );
				e.printStackTrace();
			}
		}
	}
	
	public Collection<ValueTransfer> getLastTransfers()
	{
		return lastTransfers;
	}

	public void transferValues( WsdlTestRequest sourceRequest, WsdlTestRequest targetRequest, ValueTransfer transfer) throws Exception
	{
		XmlObject previous = null;
		XmlObject[] sourceValues = null;
		
		String sourceType = transfer.getSourceType();
		if( sourceType.equals( ValueTransferSourceTypeConfig.REQUEST.toString() ))
			previous = XmlObject.Factory.parse( sourceRequest.getRequestContent());
		else if( sourceType.equals( ValueTransferSourceTypeConfig.RESPONSE.toString() ))
			previous = XmlObject.Factory.parse( sourceRequest.getResponseContent());
		else
		{
			Object value = PropertyUtils.getProperty( sourceRequest, sourceType.toLowerCase() );
			sourceValues = new XmlObject[] { XmlString.Factory.newValue( value ) };
		}
		
		if( sourceValues == null )
		{
			sourceValues = previous.selectPath( transfer.getSourcePath() );
			if( sourceValues.length == 0 ) 
			{
				log.info( "No match found for transfer sourcePath [" + transfer.getSourcePath() + 
						"] in transfer [" + transfer.getName() + "]");
				return;
			}
		}

		String targetType = transfer.getTargetType();
		if( targetType.equals( ValueTransferTargetTypeConfig.REQUEST.toString() ))
		{
			XmlObject next = XmlObject.Factory.parse( targetRequest.getRequestContent());
		   XmlObject[] destValues = next.selectPath( transfer.getTargetPath() );
		   if( destValues.length == 0 ) 
		   {
				log.info( "No match found for transfer targetPath [" + transfer.getTargetPath() + 
						"] in transfer [" + transfer.getName() + "]" );
				return;
			}
		   
		   if( sourceValues.length != destValues.length )
		   	throw new Exception( "Number of matching values must be same in source response and destination request" +
		   			"; " + sourceValues.length + ":" + destValues.length );
		   
		   for( int c = 0; c < sourceValues.length; c++ )
		   {
				if( transferValue(sourceValues[c], destValues[c], transfer))
				{
					lastTransfers.add( transfer );
				}
		   }
		   
		   if( canceled ) 
				return;
		   
		   targetRequest.setRequest( next.xmlText() );
		}
		else 
		{
			XmlObject source = sourceValues[0];
			
			Node sourceNode = source.getDomNode();
			short sourceNodeType = sourceNode.getNodeType();
			
			if( sourceNodeType == Node.DOCUMENT_FRAGMENT_NODE )
			{
				sourceNode = ((DocumentFragment) sourceNode ).getFirstChild();
				if( sourceNode != null )
					sourceNodeType = sourceNode.getNodeType();
				else
					throw new Exception( "Missing source value for " + sourceType );
			}
			
			String value = null;
			if( sourceNodeType == Node.TEXT_NODE || sourceNodeType == Node.ATTRIBUTE_NODE )
			{
				value = sourceNode.getNodeValue();
			}
			else if( sourceNodeType == Node.ELEMENT_NODE )
			{
				value = XmlUtils.getElementText( (Element) sourceNode );
			}
			
			if( value != null )
			{
				PropertyUtils.setProperty( targetRequest, targetType.toLowerCase(), value );
				lastTransfers.add( transfer );
			}
		}
	}
	
	private boolean transferValue(XmlObject source, XmlObject dest, ValueTransfer transfer) throws Exception
	{
		// just copy if nodes are of same type
		Node destNode = dest.getDomNode();
		Node sourceNode = source.getDomNode();
		short destNodeType = destNode.getNodeType();
		short sourceNodeType = sourceNode.getNodeType();
		
		if( sourceNodeType == Node.DOCUMENT_FRAGMENT_NODE )
		{
			sourceNode = ((DocumentFragment) sourceNode ).getFirstChild();
			if( sourceNode != null )
				sourceNodeType = sourceNode.getNodeType();
			else
				throw new Exception( "Missing source value for " + source );
		}
		
		// same type of node?
		if( destNodeType == sourceNodeType )
		{
			dest.set( source.copy() );
		}
		// text to attribute?
		else if( (sourceNodeType == Node.TEXT_NODE && destNodeType == Node.ATTRIBUTE_NODE) ||
				(sourceNodeType == Node.ATTRIBUTE_NODE && destNodeType == Node.TEXT_NODE) )
		{
			destNode.setNodeValue( sourceNode.getNodeValue() );
		}
		else if( sourceNodeType == Node.ELEMENT_NODE && 
				destNodeType == Node.ATTRIBUTE_NODE || destNodeType == Node.TEXT_NODE )
		{
			destNode.setNodeValue( XmlUtils.getElementText( (Element) sourceNode ));
		}
		else if( destNodeType == Node.ELEMENT_NODE && 
				sourceNodeType == Node.ATTRIBUTE_NODE || sourceNodeType == Node.TEXT_NODE )
		{
			// hmm.. not sure xmlbeans handles this ok
			XmlUtils.setElementText( (Element) destNode, sourceNode.getNodeValue() );
		}
		else return false;
		
		return true;
	}

	public PanelBuilder getPanelBuilder()
	{
		if( panelBuilder == null )
			panelBuilder = new TransferResponseValuesTestStepPanelBuilder( this );
		
		return panelBuilder;
	}

	public boolean cancel()
	{
		canceled = true;
		return canceled;
	}
	
	public int getTransferCount()
	{
		return transferStepConfig.sizeOfTransfersArray();
	}
	
	public ValueTransfer getTransferAt( int index )
	{
		return transfers.get( index );
	}
	
	public ValueTransfer addTransfer( String name )
	{
		ValueTransfer transfer = new ValueTransfer( transferStepConfig.addNewTransfers());
		transfer.setName( name );
		transfers.add( transfer );
		return transfer;
	}
	
	public void removeTransferAt( int index )
	{
		lastTransfers.remove( transfers.get( index ));
		transfers.remove( index );
		transferStepConfig.removeTransfers( index );
	}
	
	public class ValueTransfer
	{
		private ValueTransferConfig config;

		private ValueTransfer( ValueTransferConfig config )
		{
			this.config = config;
			
			if( config.getSourceType() == null )
				config.setSourceType( ValueTransferSourceTypeConfig.RESPONSE.toString() );
			if( config.getTargetType() == null )
				config.setTargetType( ValueTransferTargetTypeConfig.REQUEST.toString() );
		}
		
		private void setConfig(ValueTransferConfig config)
		{
			this.config = config;
		}

		public String getSourcePath()
		{
			return config.getSourcePath();
		}
		
		public String getTargetPath()
		{
			return config.getTargetPath();
		}
		
		public String getSourceType()
		{
			return config.getSourceType();
		}
		
		public void setSourceType( String type )
		{
			config.setSourceType( type );
		}
		
		public String getTargetType()
		{
			return config.getTargetType();
		}
		
		public void setTargetType( String type )
		{
			config.setTargetType( type );
		}
		
		public String getName()
		{
			return config.getName();
		}
		
		public void setSourcePath( String path )
		{
			config.setSourcePath( path );
		}
		
		public void setTargetPath( String path )
		{
			config.setTargetPath( path );
		}
		
		public void setName( String name )
		{
			config.setName( name );
		}
	}
}
