/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.teststeps;

import java.awt.event.ActionEvent;
import javax.swing.AbstractAction;
import javax.swing.ImageIcon;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.config.TestStepConfig;
import com.eviware.soapui.impl.wsdl.WsdlTestCase;
import com.eviware.soapui.impl.wsdl.WsdlTestStep;
import com.eviware.soapui.model.ModelItem;
import com.eviware.soapui.model.testsuite.TestRunner;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.XmlObjectConfigurationBuilder;
import com.eviware.soapui.support.XmlObjectConfigurationReader;

public class WsdlDelayTestStep extends WsdlTestStep
{
	private static final int DEFAULT_DELAY = 1000;
	private static final int DELAY_CHUNK = 500;
	private int delay = WsdlDelayTestStep.DEFAULT_DELAY;
	private int timeWaited = 0;
	private boolean canceled;
	private ImageIcon waitIcon;

	public WsdlDelayTestStep(WsdlTestCase testCase, TestStepConfig config)
	{
		super(testCase, config);
		
		if( config.getConfig() == null )
		{
			saveDelay( config );
		}
		else
		{
			readConfig(config);
		}
		
		addAction( new SetWaitTimeAction() );
		
		waitIcon = SoapUI.createImageIcon("/wait.gif");
	}

	private void readConfig(TestStepConfig config)
	{
		XmlObjectConfigurationReader reader = new XmlObjectConfigurationReader( config.getConfig() );
		delay = reader.readInt( "delay", 1000 );
	}
	
	public String getName()
	{
		return "Delay for " + (delay-timeWaited) + "ms";
	}
	
	public ImageIcon getIcon()
	{
		return waitIcon;
	}

	private void saveDelay(TestStepConfig config)
	{
		XmlObjectConfigurationBuilder builder = new XmlObjectConfigurationBuilder();
		builder.add( "delay", delay );
		config.setConfig( builder.finish() );
	}
	
	protected void resetConfigOnMove(TestStepConfig config)
	{
		super.resetConfigOnMove( config );
		readConfig( config );
	}

	public void run(TestRunner testRunner)
	{
		String oldName = getName();
		
		try
		{
			canceled = false;
			
			// sleep in chunks for canceling 
			for( timeWaited = 0; !canceled && timeWaited < delay; timeWaited += DELAY_CHUNK )
			{
				if( timeWaited % 1000 == 0 )
				{
					notifyPropertyChanged( ModelItem.NAME_PROPERTY, oldName, getName() );
					oldName = getName();
				}
				
				if( timeWaited <= delay-DELAY_CHUNK )
					Thread.sleep( DELAY_CHUNK );
				else 
					Thread.sleep( delay % DELAY_CHUNK );
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}
		
		timeWaited = 0;
		notifyPropertyChanged( ModelItem.NAME_PROPERTY, oldName, getName() );
	}

	public boolean cancel()
	{
		canceled = true;
		return true;
	}
	
	private class SetWaitTimeAction extends AbstractAction
	{
		public SetWaitTimeAction()
		{
			super( "Set Delay Time" );
		}
		
		public void actionPerformed(ActionEvent e)
		{
			String value = UISupport.prompt( "Specify delay in milliseconds", "Set Delay", String.valueOf( delay ));
			if( value != null )
			{
				try
				{
					String oldName = getName();
					int newDelay = Integer.parseInt(value);
					if( newDelay == delay )
						return;
					
					delay = newDelay;
					saveDelay( getConfig() );
				   notifyPropertyChanged( ModelItem.NAME_PROPERTY, oldName, getName() );
				}
				catch (Exception e1)
				{
				}
			}
		}
	}

}
