/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.teststeps;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.List;

import javax.swing.ImageIcon;

import org.apache.log4j.Logger;
import org.apache.xmlbeans.XmlBoolean;
import org.apache.xmlbeans.XmlObject;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.config.GotoConditionConfig;
import com.eviware.soapui.config.GotoConditionTypeConfig;
import com.eviware.soapui.config.GotoStepConfig;
import com.eviware.soapui.config.TestStepConfig;
import com.eviware.soapui.impl.wsdl.WsdlTestCase;
import com.eviware.soapui.impl.wsdl.WsdlTestStep;
import com.eviware.soapui.impl.wsdl.panels.testcase.GotoStepPanelBuilder;
import com.eviware.soapui.model.PanelBuilder;
import com.eviware.soapui.model.testsuite.TestRunner;
import com.eviware.soapui.model.testsuite.TestStep;

public class WsdlGotoTestStep extends WsdlTestStep
{
	private GotoStepConfig gotoStepConfig;
	private List<GotoCondition> conditions = new ArrayList<GotoCondition>();
	private boolean canceled;
	
	private final static Logger log = Logger.getLogger( WsdlGotoTestStep.class );
	private GotoStepPanelBuilder gotoStepPanelBuilder;
	private ImageIcon gotoIcon;
	private GotoCondition lastMatchedCondition;
	
	public WsdlGotoTestStep(WsdlTestCase testCase, TestStepConfig config)
	{
		super(testCase, config);
		
		if( config.getConfig() == null )
		{
			gotoStepConfig = (GotoStepConfig) config.addNewConfig().changeType( GotoStepConfig.type );
		}
		else
		{
			gotoStepConfig = (GotoStepConfig) config.getConfig().changeType(GotoStepConfig.type);
			for( int c = 0; c < gotoStepConfig.sizeOfConditionArray(); c++ )
			{
				conditions.add( new GotoCondition( gotoStepConfig.getConditionArray( c )));
			}	
		}
		
		gotoIcon = SoapUI.createImageIcon("/goto.gif");
	}

	protected void resetConfigOnMove(TestStepConfig config)
	{
		super.resetConfigOnMove( config );
		
		gotoStepConfig = (GotoStepConfig) config.getConfig().changeType(GotoStepConfig.type);
		for( int c = 0; c < gotoStepConfig.sizeOfConditionArray(); c++ )
		{
			conditions.get( c ).setConfig( gotoStepConfig.getConditionArray( c ));
		}	
	}

	public ImageIcon getIcon()
	{
		return gotoIcon;
	}

	public void run(TestRunner runner)
	{
		canceled = false;
		lastMatchedCondition = null;
		
		WsdlTestRequestStep previousStep = (WsdlTestRequestStep) getTestCase().findPreviousStepOfType( 
				this, WsdlTestRequestStep.class );
		
		if( previousStep == null )
		{
			log.error( "Failed to find previous request step from [" + getName() + "]" );
			return;
		}
		
		int target = runConditions( previousStep );
		if( target != -1 )
			runner.gotoStep( target );
	}

	public int runConditions(WsdlTestRequestStep previousStep)
	{
		for( GotoCondition condition : conditions )
		{
			if( canceled )
				break;
			
			try
			{
				int target = condition.evaluate( previousStep );
				if( target != -1 )
				{
					lastMatchedCondition = condition;
					return target;
				}
			}
			catch (Exception e)
			{
				log.error( "Error making condition " + condition.getName() + "; " + e );
			}
		}
		
		return -1;
	}

	public boolean cancel()
	{
		canceled = true;
		return canceled;
	}
	
	public int getConditionCount()
	{
		return conditions.size();
	}
	
	public GotoCondition getConditionAt( int index )
	{
		return conditions.get( index );
	}
	
	public GotoCondition addCondition( String name )
	{
		GotoCondition condition = new GotoCondition( gotoStepConfig.addNewCondition());
		condition.setName( name );
		condition.setType( GotoConditionTypeConfig.XPATH.toString() );
		conditions.add( condition );
		return condition;
	}
	
	public PanelBuilder getPanelBuilder()
	{
		if( gotoStepPanelBuilder == null )
			gotoStepPanelBuilder = new GotoStepPanelBuilder( this );
		
		return gotoStepPanelBuilder;
	}

	public void removeConditionAt( int index )
	{
		if( lastMatchedCondition == conditions.get( index ))
			lastMatchedCondition = null;
		
		conditions.remove( index );
		gotoStepConfig.removeCondition( index );
	}
	
	public class GotoCondition implements PropertyChangeListener
	{
		public final static String TARGET_STEP_PROPERTY = "target_step";
		
		private GotoConditionConfig conditionConfig;
		private TestStep currentStep;
		private PropertyChangeSupport propertyChangeSupport = new PropertyChangeSupport( this );

		public GotoCondition(GotoConditionConfig conditionConfig)
		{
			this.conditionConfig = conditionConfig;
			initListeners();
		}
		
		public void addPropertyChangeListener( String propertyName, PropertyChangeListener listener )
		{
			propertyChangeSupport.addPropertyChangeListener( propertyName, listener );
		}

		public void addPropertyChangeListener( PropertyChangeListener listener )
		{
			propertyChangeSupport.addPropertyChangeListener( listener );
		}
		
		public void removePropertyChangeListener( String propertyName, PropertyChangeListener listener )
		{
			propertyChangeSupport.removePropertyChangeListener( propertyName, listener );
		}

		public void removePropertyChangeListener( PropertyChangeListener listener )
		{
			propertyChangeSupport.removePropertyChangeListener( listener );
		}
		
		private void initListeners()
		{
			if( currentStep != null )
				currentStep.removePropertyChangeListener( this );
			
			if( getTargetStep() != null )
			{
				int index = getTestCase().getTestStepIndexByName( getTargetStep() );
				if( index != -1 )
				{
					currentStep = getTestCase().getTestStepAt( index );
					currentStep.addPropertyChangeListener( TestStep.NAME_PROPERTY, this );
				}
			}
		}
	
		public int evaluate(WsdlTestRequestStep previousStep) throws Exception
		{
			if( getExpression() == null || getExpression().trim().length() == 0 )
				throw new Exception( "Missing expression in condition [" + getName() + "]" );
			
			if( getTargetStep() == null || getTargetStep().trim().length() == 0 )
				throw new Exception( "Missing target step in condition [" + getName() + "]" );
			
			if( getType().equals( GotoConditionTypeConfig.XPATH.toString() ))
			{
				XmlObject xmlObject = XmlObject.Factory.parse( previousStep.getTestRequest().getResponseContent());
				XmlObject[] selectPath = xmlObject.selectPath( getExpression() );
				if( selectPath.length == 1 && selectPath[0] instanceof XmlBoolean )
				{
					if( ((XmlBoolean)selectPath[0]).getBooleanValue() )
					{
						return getTestCase().getTestStepIndexByName( getTargetStep() );
					}
				}
			}
			else
			{
				log.error( "Unkown condition type: " + getType() );
			}
			
			return -1;
		}

		protected void setConfig(GotoConditionConfig conditionConfig)
		{
			this.conditionConfig = conditionConfig;
		}

		public String getType()
		{
			return conditionConfig.getType();
		}
		
		public String getName()
		{
			return conditionConfig.getName();
		}

		public String getExpression()
		{
			return conditionConfig.getExpression();
		}
		
		public String getTargetStep()
		{
			return conditionConfig.getTargetStep();
		}
		
		public void setType( String type )
		{
			conditionConfig.setType( type );
		}

		public void setName( String name )
		{
			conditionConfig.setName( name );
		}
		
		public void setExpression( String expression )
		{
			conditionConfig.setExpression( expression );
		}
		
		public void setTargetStep( String targetStep )
		{
			String oldStep = getTargetStep();
			conditionConfig.setTargetStep( targetStep );
			initListeners();
			propertyChangeSupport.firePropertyChange( TARGET_STEP_PROPERTY, oldStep, targetStep );
		}

		public void propertyChange(PropertyChangeEvent evt)
		{
			conditionConfig.setTargetStep( evt.getNewValue().toString() );
			propertyChangeSupport.firePropertyChange( TARGET_STEP_PROPERTY, evt.getOldValue(), evt.getNewValue() );
		}
	}

	public GotoCondition getLastMatchedCondition()
	{
		return lastMatchedCondition;
	}
}
