/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.teststeps;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.swing.ImageIcon;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.config.CallConfig;
import com.eviware.soapui.config.RequestAssertionConfig;
import com.eviware.soapui.impl.wsdl.WsdlRequest;
import com.eviware.soapui.impl.wsdl.actions.request.DeleteRequestAction;
import com.eviware.soapui.impl.wsdl.actions.request.RenameRequestAction;
import com.eviware.soapui.impl.wsdl.panels.request.WsdlTestRequestPanelBuilder;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlAssertion.AssertionStatus;
import com.eviware.soapui.impl.wsdl.teststeps.actions.AddAssertionAction;
import com.eviware.soapui.impl.wsdl.teststeps.assertions.SchemaComplianceAssertion;
import com.eviware.soapui.impl.wsdl.teststeps.assertions.SimpleContainsAssertion;
import com.eviware.soapui.impl.wsdl.teststeps.assertions.SimpleNotContainsAssertion;
import com.eviware.soapui.impl.wsdl.teststeps.assertions.SoapFaultAssertion;
import com.eviware.soapui.impl.wsdl.teststeps.assertions.XPathContainsAssertion;
import com.eviware.soapui.model.PanelBuilder;
import com.eviware.soapui.model.iface.Operation;
import com.eviware.soapui.model.testsuite.TestCase;

/**
 * WsdlRequest extension that adds WsdlAssertions
 * 
 * @author Ole.Matzura
 */

public class WsdlTestRequest extends WsdlRequest implements PropertyChangeListener
{
   public static final String RESPONSE_PROPERTY = WsdlTestRequest.class.getName() + "@response";
   public static final String STATUS_PROPERTY = WsdlTestRequest.class.getName() + "@status";
   
   private List<WsdlAssertion> assertions = new ArrayList<WsdlAssertion>();
   private Map<String,Class<? extends WsdlAssertion> > availableAssertions = new HashMap<String,Class<? extends WsdlAssertion> >();
   private final static Log log = LogFactory.getLog( WsdlTestRequest.class );
   private ImageIcon validRequestIcon;
   private ImageIcon failedRequestIcon;
   private ImageIcon unknownRequestIcon;
   private WsdlAssertion.AssertionStatus currentStatus;
   private final WsdlTestRequestStep testStep;
   private List<WsdlTestRequestListener> testRequestListeners = new ArrayList<WsdlTestRequestListener>();
   
   public WsdlTestRequest( Operation operation, CallConfig callConfig, WsdlTestRequestStep testStep )
   {
      super( operation, callConfig );
      this.testStep = testStep;
      
      initAssertions();
   }
   
   protected PanelBuilder initPanelBuilder()
   {
      return new WsdlTestRequestPanelBuilder( this );
   }
   
   public TestCase getTestCase()
   {
      return testStep.getTestCase();
   }

   public WsdlTestRequestStep getRequestStep()
   {
      return testStep;
   }
   
   protected void initIcons()
   {
      super.initIcons();
      
      validRequestIcon = SoapUI.createImageIcon("/valid_request.gif");
      failedRequestIcon = SoapUI.createImageIcon("/invalid_request.gif");
      unknownRequestIcon = SoapUI.createImageIcon("/unknown_request.gif");
   }
   
   protected void initActions()
   {
      addAction( new RenameRequestAction( this ));
      addAction( new DeleteRequestAction( this ));
      addAction( new AddAssertionAction( this ));
   }

   private void initAssertions()
   {
      availableAssertions.put( "Schema Compliance", SchemaComplianceAssertion.class );
      availableAssertions.put( "Simple Contains", SimpleContainsAssertion.class );
      availableAssertions.put( "Simple NotContains", SimpleNotContainsAssertion.class );
      availableAssertions.put( "XPath Match", XPathContainsAssertion.class );
      availableAssertions.put( "SOAP Fault Assertion", SoapFaultAssertion.class );
      
      RequestAssertionConfig[] assertionConfigs = getConfig().getAssertionArray();
      for (int i = 0; i < assertionConfigs.length; i++)
      {
         RequestAssertionConfig config = assertionConfigs[i];
         String type = config.getType();
         
         if( availableAssertions.containsKey( type ))
         {
            addWsdlAssertion(config, type);
         }
      }
   }

	private WsdlAssertion addWsdlAssertion(RequestAssertionConfig config, String type) 
	{
		try
		{
		   Class<? extends WsdlAssertion> clazz = availableAssertions.get(type);
		   Constructor<? extends WsdlAssertion> ctor = clazz.getConstructor(new Class[] { RequestAssertionConfig.class, WsdlTestRequest.class });

		   WsdlAssertion assertion = (WsdlAssertion) ctor.newInstance( config, this );
		   assertions.add(assertion);
		   assertion.addPropertyChangeListener( this);
		   
		   return assertion;
		}
		catch (Exception e)
		{
		   e.printStackTrace();
		   return null;
		}
	}
   
   public int getAssertionCount()
   {
      return assertions.size();
   }

   public WsdlAssertion getAssertionAt(int c)
   {
      return assertions.get( c );
   }

   public void setResponse(String response)
   {
      super.setResponse( response );
      
      assertRequest();
   }

   private void assertRequest() 
   {
   	PropertyChangeNotifier notifier = new PropertyChangeNotifier();
      
      // assert!
      for (Iterator<WsdlAssertion> iter = assertions.iterator(); iter.hasNext();)
      {
         iter.next().assertResponse( this );
      }

      notifier.notifyChange();
   }
   
   private class PropertyChangeNotifier
   {
	   private AssertionStatus oldStatus;
	   private ImageIcon oldIcon;

	   public PropertyChangeNotifier()
	   {
		   oldStatus = getAssertionStatus();
		   oldIcon = getIcon();
	   }
	   
	   public void notifyChange()
	   {
		   AssertionStatus newStatus = getAssertionStatus();
	      ImageIcon newIcon = getIcon();
	      
	      if( oldStatus != newStatus )
	         notifyPropertyChanged( STATUS_PROPERTY, oldStatus, newStatus );
	      
	      if( oldIcon != newIcon )
	         notifyPropertyChanged( ICON_PROPERTY, oldIcon, getIcon() );
	   }
   }

   public String[] getAvailableAssertions()
   {
      return availableAssertions.keySet().toArray( new String[availableAssertions.size()] );
   }

   public WsdlAssertion addAssertion(String assertionName)
   {
      if( !availableAssertions.containsKey( assertionName )) return null;
      PropertyChangeNotifier notifier = new PropertyChangeNotifier();
    
      RequestAssertionConfig assertionConfig = getConfig().addNewAssertion();
      assertionConfig.setType( assertionName );
      
      try
      {
      	WsdlAssertion assertion = addWsdlAssertion( assertionConfig, assertionName );
      	notifyAssertionAdded( assertion );
      	
         if( getResponseContent() != null )
         {
            assertion.assertResponse( this );
            notifier.notifyChange();
         }
         
         return assertion;
      }
      catch (Exception e)
      {
         e.printStackTrace();
         return null;
      }
   }

   private void notifyAssertionAdded(WsdlAssertion assertion)
	{
   	WsdlTestRequestListener [] listeners = testRequestListeners.toArray( 
   			new WsdlTestRequestListener[testRequestListeners.size()] );
   	
   	for( int c = 0; c < listeners.length; c++ )
   	{
   		listeners[c].assertionAdded( assertion );
   	}
	}

   private void notifyAssertionRemoved( WsdlAssertion assertion)
	{
   	WsdlTestRequestListener [] listeners = testRequestListeners.toArray( 
   			new WsdlTestRequestListener[testRequestListeners.size()] );
   	
   	for( int c = 0; c < listeners.length; c++ )
   	{
   		listeners[c].assertionRemoved( assertion );
   	}
	}

   
	public void removeAssertion(WsdlAssertion assertion)
   {
      int ix = assertions.indexOf( assertion );
      if( ix == -1 ) 
      {
      	throw new RuntimeException( "assertion [" + assertion.getName() + "] not available " +
      			"in test request [" + getName() + "]" );
      }
      
      PropertyChangeNotifier notifier = new PropertyChangeNotifier();
      
      assertion.removePropertyChangeListener( this );
      notifyAssertionRemoved( assertion );
      
      assertions.remove( ix );
      getConfig().removeAssertion( ix );
      
      notifier.notifyChange();
   }

   public AssertionStatus getAssertionStatus()
   {
      currentStatus = AssertionStatus.NONE;

      int c = getAssertionCount();
      if( c == 0 ) return currentStatus;
      
      currentStatus = AssertionStatus.VALID;
      
      for( c = 0; c < getAssertionCount(); c++ )
      {
         AssertionStatus status = getAssertionAt( c ).getStatus();
         if( status == AssertionStatus.FAILED )
         {
            currentStatus = AssertionStatus.FAILED;
            break;
         }
         else if( status == AssertionStatus.UNKNOWN )
         {
            currentStatus = AssertionStatus.UNKNOWN;
            break;
         }
      }
      
      return currentStatus;
   }

   public ImageIcon getIcon()
   {
   	ImageIcon icon = getIconManager().getIcon();
      if( icon == getIconManager().getRequestIcon() )
      {
         AssertionStatus status = getAssertionStatus();
         if( status == AssertionStatus.VALID ) return validRequestIcon; 
         else if( status == AssertionStatus.FAILED ) return failedRequestIcon; 
         else if( status == AssertionStatus.UNKNOWN ) return unknownRequestIcon; 
      }
      
      return icon;
   }

   public void propertyChange(PropertyChangeEvent evt)
   {
	  if( evt.getPropertyName().equals( WsdlAssertion.CONFIGURATION_PROPERTY ))
		  assertRequest();
   }

   public void addWsdlTestRequestListener( WsdlTestRequestListener listener )
   {
   	testRequestListeners.add( listener );
   }
   
   public void removeWsdlTestRequestListener( WsdlTestRequestListener listener )
   {
   	testRequestListeners.remove( listener );
   }
   
   public interface WsdlTestRequestListener
   {
   	public void assertionAdded( WsdlAssertion assertion );
   	
   	public void assertionRemoved( WsdlAssertion assertion );
   }

	public void updateConfig(CallConfig request)
	{
		super.updateConfig(request);
		
		RequestAssertionConfig[] assertionConfigs = getConfig().getAssertionArray();
      for (int i = 0; i < assertionConfigs.length; i++)
      {
         RequestAssertionConfig config = assertionConfigs[i];
         assertions.get( i ).updateConfig( config );
      }
	}

	
}
