/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.support;

import java.awt.Color;
import java.awt.Font;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.KeyStroke;
import javax.swing.event.UndoableEditEvent;
import javax.swing.event.UndoableEditListener;
import javax.swing.text.BadLocationException;
import javax.swing.undo.CannotRedoException;
import javax.swing.undo.CannotUndoException;
import javax.swing.undo.UndoManager;

import org.syntax.jedit.DefaultInputHandler;
import org.syntax.jedit.JEditTextArea;
import org.syntax.jedit.SyntaxStyle;
import org.syntax.jedit.tokenmarker.Token;
import org.syntax.jedit.tokenmarker.XMLTokenMarker;

import com.eviware.soapui.support.actions.FindAndReplaceAction;

/**
 * JEditTextArea extension targeted specifically at XML-editing.
 * 
 * @author Ole.Matzura
 */

public class JXmlTextArea extends JEditTextArea implements
		UndoableEditListener, FocusListener
{
	
	public static final int UNDO_LIMIT = 1500;

	private UndoManager undoManager;

	private UndoAction undoAction;

	private RedoAction redoAction;

	private FindAndReplaceAction findAndReplaceAction;
	
	private boolean discardEditsOnSet = true;

	public JXmlTextArea()
	{
		getPainter().setFont(new Font("Courier", Font.PLAIN, 11));
		getPainter().setStyles(createXmlStyles());
		setTokenMarker(new XMLTokenMarker());
		setBorder(BorderFactory.createEtchedBorder());
		getDocument().addUndoableEditListener(this);
		addFocusListener(this);

		undoAction = new UndoAction();
		getInputHandler().addKeyBinding("C+Z", undoAction);
		redoAction = new RedoAction();
		getInputHandler().addKeyBinding("C+Y", redoAction);
		findAndReplaceAction = new FindAndReplaceAction( this );
		getInputHandler().addKeyBinding( "C+F", findAndReplaceAction );
		getInputHandler().addKeyBinding("A+RIGHT", new NextElementValueAction() );
		getInputHandler().addKeyBinding("A+LEFT", new PreviousElementValueAction() );
		getInputHandler().addKeyBinding("C+D", new DeleteLineAction() );
	}

	public Action getFindAndReplaceAction()
	{
		return findAndReplaceAction;
	}

	public Action getRedoAction()
	{
		return redoAction;
	}

	public Action getUndoAction()
	{
		return undoAction;
	}

	public void setText(String text)
	{
		super.setText(text);
		
		if( discardEditsOnSet && undoManager != null )
			undoManager.discardAllEdits();
	}
	
	public boolean isDiscardEditsOnSet()
	{
		return discardEditsOnSet;
	}

	public void setDiscardEditsOnSet(boolean discardEditsOnSet)
	{
		this.discardEditsOnSet = discardEditsOnSet;
	}

	public UndoManager getUndoManager()
	{
		return undoManager;
	}

	public SyntaxStyle[] createXmlStyles()
	{
		SyntaxStyle[] styles = new SyntaxStyle[Token.ID_COUNT];

		styles[Token.COMMENT1] = new SyntaxStyle(Color.black, true, false);
		styles[Token.COMMENT2] = new SyntaxStyle(new Color(0x990033), true, false);
		styles[Token.KEYWORD1] = new SyntaxStyle(Color.blue, false, false);
		styles[Token.KEYWORD2] = new SyntaxStyle(Color.magenta, false, false);
		styles[Token.KEYWORD3] = new SyntaxStyle(new Color(0x009600), false,
				false);
		styles[Token.LITERAL1] = new SyntaxStyle(new Color(0x650099), false,
				false);
		styles[Token.LITERAL2] = new SyntaxStyle(new Color(0x650099), false, true);
		styles[Token.LABEL] = new SyntaxStyle(new Color(0x990033), false, true);
		styles[Token.OPERATOR] = new SyntaxStyle(Color.black, false, true);
		styles[Token.INVALID] = new SyntaxStyle(Color.red, false, true);

		return styles;
	}

	private void createUndoMananger()
	{
		undoManager = new UndoManager();
		undoManager.setLimit(UNDO_LIMIT);
	}
	
	/*

	private void removeUndoMananger()
	{
		if (undoManager == null)
			return;
		undoManager.end();
		undoManager = null;
	}
	
	*/

	public void focusGained(FocusEvent fe)
	{
		if (isEditable() && undoManager == null )
			createUndoMananger();
	}

	public void setEditable(boolean enabled)
	{
		super.setEditable(enabled);
		getPainter().setBackground( enabled ? Color.WHITE : new Color(220, 220, 220) );
		repaint();
	}

	public void focusLost(FocusEvent fe)
	{
		//removeUndoMananger();
	}

	public void undoableEditHappened(UndoableEditEvent e)
	{
		if (undoManager != null)
			undoManager.addEdit(e.getEdit());
	}

	private class UndoAction extends AbstractAction
	{
		public UndoAction()
		{
			super( "Undo");
			putValue( Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke( KeyEvent.VK_Z, KeyEvent.CTRL_MASK ));
		}
		
		public void actionPerformed(ActionEvent e)
		{
			if( !isEditable()  )
			{
				getToolkit().beep();
				return;
			}
			
			try
			{
				if( undoManager != null )
					undoManager.undo();
			}
			catch (CannotUndoException cue)
			{
				Toolkit.getDefaultToolkit().beep();
			}
		}
	}

	private class RedoAction extends AbstractAction
	{
		public RedoAction()
		{
			super( "Redo");
			putValue( Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke( KeyEvent.VK_Y, KeyEvent.CTRL_MASK ));
		}

		public void actionPerformed(ActionEvent e)
		{
			if( !isEditable()  )
			{
				getToolkit().beep();
				return;
			}
			
			try
			{
				if( undoManager != null )
					undoManager.redo();
			}
			catch (CannotRedoException cue)
			{
				Toolkit.getDefaultToolkit().beep();
			}
		}
	}
	
	public class NextElementValueAction implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			int pos = getCaretPosition();
			String text = getText();
			
			while( pos < text.length() )
			{
				// find ending >
				if( text.charAt( pos ) == '>' && pos < text.length()-1 && 
						(pos > 2 && !text.substring(pos-2,pos).equals( "--" )) &&
						(pos > 1 && text.charAt(pos-1) != '/' ) &&
						text.indexOf( '/', pos ) == text.indexOf( '<', pos) +1 &&
						text.lastIndexOf( '/', pos ) != text.lastIndexOf( '<', pos) +1	)
				{
					setCaretPosition( pos+1 );
					return;
				}		
				
				pos++;
			}			
			
			getToolkit().beep();
		}
	}
	
	public class PreviousElementValueAction implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			int pos = getCaretPosition()-2;
			String text = getText();
			
			while( pos > 0 )
			{
				// find ending >
				if( text.charAt( pos ) == '>' && pos < text.length()-1 && 
						(pos > 2 && !text.substring(pos-2,pos).equals( "--" )) &&
						(pos > 1 && text.charAt(pos-1) != '/' ) &&
						text.indexOf( '/', pos ) == text.indexOf( '<', pos) +1 &&
						text.lastIndexOf( '/', pos ) != text.lastIndexOf( '<', pos) +1	)
				{
					setCaretPosition( pos+1 );
					return;
				}		
				
				pos--;
			}				

			getToolkit().beep();
		}
	}

	
	public class DeleteLineAction implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			if( !isEditable() )
			{
				getToolkit().beep();
				return;
			}
			
			int caretLine = getCaretLine();
			if( caretLine == -1 ) return;
			int lineStartOffset = getLineStartOffset( caretLine);
			int lineEndOffset = getLineEndOffset( caretLine);
			
			try
			{
				int len = lineEndOffset-lineStartOffset;
				if( lineStartOffset+len >= getDocumentLength() ) len = getDocumentLength()-lineStartOffset;
				
				getDocument().remove( lineStartOffset, len );
			}
			catch (BadLocationException e1)
			{
				e1.printStackTrace();
			}
		}
	}

	public Action getCopyAction()
	{
		return DefaultInputHandler.CLIP_COPY;
	}
	
	public Action getCutAction()
	{
		return DefaultInputHandler.CLIP_CUT;
	}
	
	public Action getPasteAction()
	{
		return DefaultInputHandler.CLIP_PASTE;
	}
}
