/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.support;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStreamWriter;
import java.io.StringReader;
import java.io.StringWriter;
import java.io.Writer;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.StringTokenizer;

import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.log4j.Logger;
import org.apache.xmlbeans.XmlCursor;
import org.apache.xmlbeans.XmlException;
import org.apache.xmlbeans.XmlObject;
import org.apache.xmlbeans.XmlOptions;
import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import org.w3c.dom.traversal.DocumentTraversal;
import org.w3c.dom.traversal.NodeFilter;
import org.w3c.dom.traversal.TreeWalker;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import com.sun.org.apache.xpath.internal.XPathAPI;

/**
 * General XML-related utilities
 */

public final class XmlUtils
{
   private static DocumentBuilder documentBuilder;
   private final static Logger log = Logger.getLogger( XmlUtils.class );

   static public Document parse( InputStream in )
   {
      try
      {
         return ensureDocumentBuilder().parse( in );
      }
      catch( Exception e )
      {
      	log.error( "Error parsing InputStream; " + e.getMessage(), e );
      }

      return null;
   }

   static public Document parse( String fileName ) throws IOException
   {
      try
      {
         return  ensureDocumentBuilder().parse( fileName );
      }
      catch( SAXException e )
      {
      	log.error( "Error parsing fileName [" + fileName + "]; " + e.getMessage(), e );
      }

      return null;
   }

   static public Document parse( InputSource inputSource ) throws IOException
   {
      try
      {
         return ensureDocumentBuilder().parse( inputSource );
      }
      catch( SAXException e )
      {
      	log.error( "Error parsing InputSource; " + e.getMessage(), e );
      }

      return null;
   }


   private static DocumentBuilder ensureDocumentBuilder()
   {
      if( documentBuilder == null )
      {
         try
         {
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            dbf.setNamespaceAware( true );
				documentBuilder = dbf.newDocumentBuilder();
         }
         catch( ParserConfigurationException e )
         {
         	log.error( "Error creating DocumentBuilder; " + e.getMessage(), e );
         }
      }

      return documentBuilder;
   }

   public static void serializePretty( Document document )
   {
      try
      {
         serializePretty( document, new OutputStreamWriter( System.out ) );
      }
      catch( IOException e )
      {
         e.printStackTrace();
      }
   }

   public static void serializePretty( Document dom, Writer writer )
      throws IOException
   {
   	try
		{
			XmlObject xmlObject = XmlObject.Factory
					.parse(dom.getDocumentElement());
			serializePretty(xmlObject, writer);
		}
		catch (Exception e)
		{
			throw new IOException( e.getMessage() );
		}
   }
   
   public static void serializePretty( XmlObject xmlObject, Writer writer )
   throws IOException
	{
		try
		{
			XmlOptions options = new XmlOptions();
			options.setSavePrettyPrint();
			options.setSavePrettyPrintIndent( 3 );
			options.setSaveNoXmlDecl();
			options.setSaveAggressiveNamespaces();
			xmlObject.save(writer, options);
		}
		catch (Exception e)
		{
			throw new IOException( e.getMessage() );
		}
	}

   public static void serialize( Document dom, Writer writer )
   throws IOException
	{
   	try
		{
			XmlObject xmlObject = XmlObject.Factory
					.parse(dom.getDocumentElement());
			xmlObject.save( writer );
		}
		catch (Exception e)
		{
			throw new IOException( e.getMessage() );
		}
	}
   
   static public void setElementText( Element elm, String text )
   {
      Node node = elm.getFirstChild();
      if( node == null  )
      {
      	if( text != null)
      		elm.appendChild( elm.getOwnerDocument().createTextNode( text ) );
      }
      else if( node.getNodeType() == Node.TEXT_NODE )
      {
         if( text == null )
            node.getParentNode().removeChild( node );
         else
            node.setNodeValue( text );
      }
      else if( text != null )
      {
         Text textNode = node.getOwnerDocument().createTextNode( text );
         elm.insertBefore( textNode, elm.getFirstChild() );
      }
   }

   public static String getChildElementText( Element elm, String name )
   {
      Element child = getFirstChildElement( elm, name );
      return child == null ? null : getElementText( child );
   }

   public static Element getFirstChildElement( Element elm, String name )
   {
      NodeList nl = elm.getChildNodes();
      for( int c = 0; c < nl.getLength(); c++ )
      {
         Node node = nl.item( c );
         if( node.getNodeType() == Node.ELEMENT_NODE && node.getNodeName().equals( name ) )
            return (Element) node;
      }

      return null;
   }

   static public String getElementText( Element elm )
   {
      Node node = elm.getFirstChild();
      if( node != null && node.getNodeType() == Node.TEXT_NODE )
         return node.getNodeValue();

      return null;
   }

   public static String getChildElementText( Element elm, String name, String defaultValue )
   {
      String result = getChildElementText( elm, name );
      return result == null ? defaultValue : result;
   }

   static public String getNodeValue( Node node )
   {
      return node.getNodeType() == Node.ELEMENT_NODE ? getElementText( (Element) node ) : node.getNodeValue();
   }

   public static Node createNodeFromPath( Element modelElement, String path )
   {
      try
      {
         Document document = modelElement.getOwnerDocument();
         StringTokenizer st = new StringTokenizer( path, "/" );
         while( st.hasMoreTokens() )
         {
            String t = st.nextToken();

            if( st.hasMoreTokens() )
            {
               if( t.equals( ".." ) )
               {
                  modelElement = (Element) modelElement.getParentNode();
               }
               else
               {
                  Element elm = getFirstChildElement( modelElement, t );
                  if( elm == null )
                     modelElement = (Element) modelElement.insertBefore(
                        document.createElement( t ),
                        getFirstChildElement( modelElement, t ) );
                  else
                     modelElement = elm;
               }
            }
            else
            {
               modelElement = (Element) modelElement.insertBefore(
                  document.createElement( t ),
                  getFirstChildElement( modelElement, t ) );
            }
         }

         return modelElement;
      }
      catch( Exception e )
      {
         //  System.err.println( "Failed to create path [" + path + "]" );
      }

      return null;
   }

   public static Element addChildElement( Element element, String name, String text )
   {
      Document document = element.getOwnerDocument();
      Element result = (Element) element.appendChild( document.createElement( name ) );
      if( text != null )
         result.appendChild( document.createTextNode( text ) );

      return result;
   }

   public static void setChildElementText( Element element, String name, String text )
   {
      Element elm = getFirstChildElement( element, name );
      if( elm == null )
      {
         elm = element.getOwnerDocument().createElement( name );
         element.appendChild( elm );
      }

      setElementText( elm, text );
   }

   public static Document parseXml( String xmlString ) throws IOException
   {
      return parse( new InputSource( new StringReader( xmlString )));
   }
   
   public static void dumpParserErrors(XmlObject xmlObject)
   {
      List errors = new ArrayList();
      xmlObject.validate(new XmlOptions().setErrorListener(errors));
      for (Iterator i = errors.iterator(); i.hasNext();)
      {
         System.out.println(i.next());
      }
   }

	public static String transferValues(String source, String dest)
	{
		try
		{
			Document sourceDom = parseXml(source);
			Document destDom = parseXml(dest);
			
			TreeWalker walker = ((DocumentTraversal)sourceDom).createTreeWalker( sourceDom.getDocumentElement(),
					NodeFilter.SHOW_ELEMENT, null, true );
			
			Element elm = (Element) walker.nextNode();
			while( elm != null )
			{
				String path = getElementPath( elm );
				Node nsNode = getNSNode( elm );
				Element elm2 = (Element) XPathAPI.selectSingleNode( destDom.getDocumentElement(), path, nsNode );
				if( elm2 != null )
				{
					// transfer attributes
					NamedNodeMap attributes = elm.getAttributes();
					for( int c = 0; c < attributes.getLength(); c++ )
					{
						Attr attr = (Attr) attributes.item( c );
						elm2.setAttribute( attr.getNodeName(), attr.getNodeValue() );
					}
					
					// transfer text
					setElementText( elm2, getElementText( elm ));
				}
				
				elm = (Element) walker.nextNode();
			}

			StringWriter writer = new StringWriter();
			serialize( destDom, writer );
			return writer.toString();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
		
		return dest;
	}

	private static Node getNSNode(Node elm)
	{
		Element result = elm.getOwnerDocument().createElement("nsnode");
		
		while( elm != null )
		{
			NamedNodeMap attributes = elm.getAttributes();
			if( attributes != null )
			{
				for( int c = 0; c < attributes.getLength(); c++ )
				{
					Node attr = attributes.item( c );
					if( attr.getNodeName().startsWith( "xmlns:" ) || attr.getNodeName().equals( "xmlns") )
					{
						result.setAttribute( attr.getNodeName(), attr.getNodeValue() );
					}
				}
			}
			
			elm = elm.getParentNode();
		}
		
		return result;
	}

	/**
	 * Returns absolute xpath for specified element, ignores namespaces
	 *  
	 * @param elm the element to create for
	 * @return the elements path in its containing document
	 */
	
	public static String getElementPath(Element element)
	{
		Node elm = element;
		
		String result = elm.getNodeName() + "[" + getElementIndex( elm ) + "]";
		while( elm.getParentNode() != null && elm.getParentNode().getNodeType() != Node.DOCUMENT_NODE )
		{
			elm = elm.getParentNode();
			result = elm.getNodeName() + "[" + getElementIndex( elm ) + "]/" + result;
		}

		return "/" + result;
	}

	/**
	 * Gets the index of the specified element amongst elements with the same name
	 * 
	 * @param element the element to get for
	 * @return the index of the element, will be >= 1
	 */
	
	public static int getElementIndex(Node element)
	{
		int result = 1;
		
		Node elm = element.getPreviousSibling();
		while( elm != null )
		{
			if( elm.getNodeType() == Node.ELEMENT_NODE && elm.getNodeName().equals( element.getNodeName() ))
				result++;
			elm = elm.getPreviousSibling();
		}
		
		return result;	
	}
	
	public static String declareXPathNamespaces( String xmlString ) throws XmlException
	{
		XmlObject xml = XmlObject.Factory.parse(xmlString);
		Map<QName,String> map = new HashMap<QName,String>();
		XmlCursor cursor = xml.newCursor();
		
		while( cursor.hasNextToken() )
		{
			if( cursor.toNextToken().isNamespace() )
				map.put( cursor.getName(), cursor.getTextValue() );
		}
		
		Iterator<QName> i = map.keySet().iterator();
		int nsCnt = 0;
		
		StringBuffer buf = new StringBuffer();
		Set<String> prefixes = new HashSet<String>();
		Set<String> usedPrefixes = new HashSet<String>();
		
		while( i.hasNext() )
		{
			QName name = i.next();
			String prefix = name.getLocalPart();
			if( prefix.length() == 0 ) prefix = "ns" + (++nsCnt);
			else if( prefix.equals( "xsd") || prefix.equals( "xsi")) continue;
			
			if( prefixes.contains( prefix ))
			{
				int c = 1;
				while( usedPrefixes.contains( prefix + c )) c++;
				
				prefix = prefix + c;
			}
			else prefixes.add( prefix );
			
			buf.append( "declare namespace " );
			buf.append( prefix );
			buf.append( "='" );
			buf.append( map.get( name ));
			buf.append( "';\n");
			
			usedPrefixes.add( prefix );
		}
		
		return buf.toString();
		
	}
}

