/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.tools;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.PosixParser;
import org.apache.log4j.Logger;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.wsdl.WsdlProject;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlAssertion;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestRequest;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestRequestStep;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlAssertion.AssertionStatus;
import com.eviware.soapui.model.testsuite.TestCase;
import com.eviware.soapui.model.testsuite.TestRunListener;
import com.eviware.soapui.model.testsuite.TestRunner;
import com.eviware.soapui.model.testsuite.TestStep;
import com.eviware.soapui.model.testsuite.TestSuite;

/**
 * Standalone test-runner used from maven-plugin, can also be used from command-line (see xdocs) or
 * directly from other classes.
 * <p>
 * For standalone usage, set the project file (with setProjectFile) and other desired properties before
 * calling run</p> 
 * 
 * @author Ole.Matzura
 */

public class SoapUITestCaseRunner implements TestRunListener
{
	private String projectFile;
	private String testSuite;
	private String testCase;
	private List<WsdlAssertion> assertions = new ArrayList<WsdlAssertion>();
	private String endpoint;
	private final static Logger log = Logger.getLogger( SoapUITestCaseRunner.class );
	private long testCaseStartTime;
	private String domain;
	private String password;
	private String username;
	
	/**
	 * Runs the tests in the specified soapui project file, see soapui xdocs for details.
	 * 
	 * @param args
	 * @throws Exception
	 */

	public static void main( String [] args) throws Exception
	{
		System.out.println( "SoapUI " + SoapUI.SOAPUI_VERSION + " TestRunner");
		
		SoapUITestCaseRunner runner = new SoapUITestCaseRunner();
		
		Options options = new Options();
		options.addOption( "e", true, "Sets the endpoint" );
		options.addOption( "s", true, "Sets the testsuite" );
		options.addOption( "c", true, "Sets the testcase" );
		options.addOption( "u", true, "Sets the username" );
		options.addOption( "p", true, "Sets the password" );
		options.addOption( "d", true, "Sets the domain" );
		
		CommandLineParser parser = new PosixParser();
		CommandLine cmd = parser.parse( options, args);
		
		String[] args2 = cmd.getArgs();
		if( args2.length != 1 )
		{
			System.out.println( "Missing soapui project file.." );
			return;
		}
		
		runner.setProjectFile( args2[0] );
		
		if( cmd.hasOption( "e"))
			runner.setEndpoint( cmd.getOptionValue( "e" ) );
		
		if( cmd.hasOption( "s"))
			runner.setTestSuite( cmd.getOptionValue( "s") );

		if( cmd.hasOption( "c"))
			runner.setTestCase( cmd.getOptionValue( "c") );

		if( cmd.hasOption( "u"))
			runner.setUsername( cmd.getOptionValue( "u") );

		if( cmd.hasOption( "p"))
			runner.setPassword( cmd.getOptionValue( "p") );

		if( cmd.hasOption( "d"))
			runner.setDomain( cmd.getOptionValue( "d") );

		runner.run();
	}
	
	/**
	 * Sets the domain to use for any authentications
	 * 
	 * @param domain the domain to use for any authentications
	 */
	
	public void setDomain(String domain)
	{
		log.info( "Setting domain to [" + domain + "]" );
		this.domain = domain;
	}

	/**
	 * Sets the password to use for any authentications
	 * 
	 * @param domain the password to use for any authentications
	 */
	
	public void setPassword(String password)
	{
		log.info( "Setting password to [" + password + "]" );
		this.password = password;
	}

	/**
	 * Sets the username to use for any authentications
	 * 
	 * @param domain the username to use for any authentications
	 */
	
	public void setUsername(String username)
	{
		log.info( "Setting username to [" + username + "]" );
		this.username = username;
	}
	
	/**
	 * Runs the testcases as configured with setXXX methods
	 * 
	 * @throws Exception thrown if any tests fail
	 */

	public void run() throws Exception
	{
		assertions.clear();
		
		if( !new File( projectFile ).exists() )
			throw new Exception( "soapui project file [" + projectFile + "] not found" );
		
		WsdlProject project = new WsdlProject( projectFile );
		log.info( "Running soapui tests in project [" + project.getName() + "]" );
		
		for( int c = 0; c < project.getTestSuiteCount(); c++ )
		{
			if( testSuite == null ||
				 project.getTestSuiteAt( c ).getName().equalsIgnoreCase( testSuite ))
			{
				runSuite( project.getTestSuiteAt( c ));
			}
		}
		
		if( assertions.size() > 0 )
		{
			StringBuffer buf = new StringBuffer();
			for( int c = 0; c < assertions.size(); c++ )
			{
				WsdlAssertion assertion = assertions.get( c );
				buf.append( assertion.getName() + " in [" + assertion.getRequest().getName() + "] failed;\n" );
				buf.append( assertion.getErrors() + "\n" );
				buf.append( "Request: " + assertion.getRequest().getRequestContent() + "\n" );
				buf.append( "Response: " + ((WsdlTestRequest)assertion.getRequest()).getResponseContent() + "\n" );
			}
			
			throw new Exception( buf.toString() );
		}
	}

	/**
	 * Run tests in the specified TestSuite
	 *
	 * @param suite the TestSuite to run
	 */
	
	public void runSuite(TestSuite suite)
	{
		log.info(( "Running soapui suite [" + suite.getName() + "]"));
		long start = System.currentTimeMillis();
		for( int c = 0; c < suite.getTestCaseCount(); c++ )
		{
			String name = suite.getTestCaseAt( c ).getName();
			if( testCase == null || 
				 name.equalsIgnoreCase( testCase ))
			{
				runTestCase( suite.getTestCaseAt( c ));
			}
			else
				System.out.println( "Skipping testcase [" + name + "], filter is [" + testCase + "]");
		}
		log.info( "soapui suite [" + suite.getName() + "] finished in " + (System.currentTimeMillis()-start) + "ms" );
	}

	/**
	 * Runs the specified TestCase
	 * 
	 * @param testCase the testcase to run
	 */
	
	private void runTestCase(TestCase testCase)
	{
		testCase.addTestRunListener( this );
		testCase.run().waitUntilFinished();
	}

	/**
	 * Sets the soapui project file containing the tests to run
	 * 
	 * @param projectFile the soapui project file containing the tests to run
	 */
	
	public void setProjectFile(String projectFile)
	{
		log.info( "setting projectFile to [" + projectFile + "]" );
		this.projectFile = projectFile;
	}

	/**
	 * Sets the testcase to run
	 * 
	 * @param testCase the testcase to run
	 */
	
	public void setTestCase(String testCase)
	{
		log.info( "setting testCase to [" + testCase + "]" );
      this.testCase = testCase;
	}
	
	/**
	 * Sets the endpoint to use for all test requests
	 * 
	 * @param endpoint the endpoint to use for all test requests
	 */
	
	public void setEndpoint(String endpoint)
	{
		log.info( "setting test endpoint to [" + endpoint+ "]" );
		this.endpoint = endpoint.trim();
	}
	
	/**
	 * Sets the TestSuite to run. If not set all TestSuites in the specified project file are run
	 * 
	 * @param testSuite the testSuite to run.
	 */

	public void setTestSuite(String testSuite)
	{
	   log.info( "setting testSuite to [" + testSuite + "]" );
		this.testSuite = testSuite;
	}
	
	public boolean beforeRun(TestRunner testRunner)
	{
		testCaseStartTime = System.currentTimeMillis();
		log.info( "Running soapui testcase [" + testRunner.getTestCase().getName() + "]");
		return true;
	}

	public boolean beforeStep(TestRunner testRunner)
	{
		TestStep currentStep = testRunner.getCurrentStep();
		log.info( "runing step [" + currentStep.getName() + "]" );
		
		if( currentStep instanceof WsdlTestRequestStep )
		{
			WsdlTestRequestStep requestStep = (WsdlTestRequestStep) currentStep;
			if( endpoint != null && endpoint.length() > 0 )
			{
				requestStep.getTestRequest().setEndpoint( endpoint );
			}
			
			if( username != null && username.length() > 0 )
			{
				requestStep.getTestRequest().setUsername( username );
			}
			
			if( password != null && password.length() > 0 )
			{
				requestStep.getTestRequest().setPassword( password );
			}
			
			if( domain != null && domain.length() > 0 )
			{
				requestStep.getTestRequest().setDomain( domain );
			}
		}
		
		return true;
	}

	public boolean afterStep(TestRunner testRunner)
	{
		TestStep currentStep = testRunner.getCurrentStep();
		
		if( currentStep instanceof WsdlTestRequestStep )
		{
			WsdlTestRequestStep requestStep = (WsdlTestRequestStep) currentStep;
			for( int c = 0; c < requestStep.getAssertionCount(); c++ )
			{
				WsdlAssertion assertion = requestStep.getAssertionAt( c );
				System.out.println( "Assertion [" + assertion.getName() + "] has status " + assertion.getStatus());
				if( assertion.getStatus() != AssertionStatus.VALID )
				{
					System.out.println( "ASSERTION FAILED -> " + assertion.getErrors());
					assertions.add( assertion );
				}
			}
		}	
		
		return true;
	}

	public void afterRun(TestRunner testRunner)
	{
		long end = System.currentTimeMillis();
		log.info( "Finished running soapui testcase [" + testRunner.getTestCase().getName() + "], time taken = " + 
				(end-testCaseStartTime) + "ms" );
	}
}