/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.model.impl.wsdl.assertions;

import java.io.BufferedReader;
import java.io.InputStreamReader;

import junit.framework.TestCase;

import org.apache.xmlbeans.XmlException;
import org.apache.xmlbeans.XmlObject;

import com.eviware.soapui.config.RequestAssertionConfig;
import com.eviware.soapui.impl.wsdl.teststeps.assertions.XPathContainsAssertion;

public class XPathContainsAssertionTestCase extends TestCase
{
	private String testResponse;
	private XPathContainsAssertion assertion;
	private String testBody;

	protected void setUp() throws Exception
	{
		testResponse = readResource( "/testResponse.xml" );
		testBody = readResource( "/testBody.xml" );
		assertion = new XPathContainsAssertion( RequestAssertionConfig.Factory.newInstance(), null );
	}

	public void testCreate() throws Exception
	{
		RequestAssertionConfig config = createConfig( "testPath", "testContent" );
		
		XPathContainsAssertion assertion = new XPathContainsAssertion( config, null );
		
		assertEquals( "testPath", assertion.getPath() );
		assertEquals( "testContent", assertion.getContent() );
		
		XmlObject conf = assertion.createConfiguration();
		String str = conf.xmlText();

      assertEquals( "<xml-fragment><path>testPath</path><content>testContent</content></xml-fragment>", str );
	}
	
	public void testFullContentMatch() throws Exception
	{
		assertion.setPath( "/" );
		assertion.setContent( testResponse );
		
		assertNotNull( assertion.assertResponse( testResponse ));
	}
	
   public void testFullBodyMatch() throws Exception
	{
		assertion.setPath( "declare namespace urn='urn:schema:v1:companyservice:applications:bis.bonnier.se';" +
               "//urn:searchResponse" );
		
		assertion.setContent( testBody );
		
		assertNotNull( assertion.assertResponse( testResponse ));
	}
   
   public void testAttributeMatch() throws Exception
	{
		assertion.setPath( "declare namespace env='http://schemas.xmlsoap.org/soap/envelope/';" +
				"declare namespace urn='urn:schema:v1:companyservice:applications:bis.bonnier.se';" +
				"declare namespace urn1='urn:v1:companysearch:common:bis.bonnier.se';" +
               "/env:Envelope/env:Body/urn:searchResponse/urn1:searchResult/@hitCount" );
		assertion.setContent( "131" );
		
		assertNotNull( assertion.assertResponse( testResponse ));
	}
   
   public void testElementMatch() throws Exception
	{
		assertion.setPath( "declare namespace urn='urn:schema:v1:companyservice:applications:bis.bonnier.se';" +
				"declare namespace urn1='urn:v1:companysearch:common:bis.bonnier.se';" +
               "//urn:searchResponse/urn1:searchResult/company[2]/companyName" );
		assertion.setContent( "<companyName>Bonnier Otto Karl Adam</companyName>" );
		
		assertNotNull( assertion.assertResponse( testResponse ));
	}

   public void testElementTextMatch() throws Exception
	{
		assertion.setPath( "declare namespace env='http://schemas.xmlsoap.org/soap/envelope/';" +
				"declare namespace urn='urn:schema:v1:companyservice:applications:bis.bonnier.se';" +
				"declare namespace urn1='urn:v1:companysearch:common:bis.bonnier.se';" +
               "/env:Envelope/env:Body/urn:searchResponse/urn1:searchResult/company[2]/companyName/text()" );
		assertion.setContent( "Bonnier Otto Karl Adam" );
		
		assertNotNull( assertion.assertResponse( testResponse ));
	}
   
   public void testFragmentMatch() throws Exception
   {
		assertion.setPath( "declare namespace urn='urn:schema:v1:companyservice:applications:bis.bonnier.se';" +
				"declare namespace urn1='urn:v1:companysearch:common:bis.bonnier.se';" +
               "//urn:searchResponse/urn1:searchResult/company[4]" );
		assertion.setContent( readResource( "/testFragment.xml") );
		
		assertNotNull( assertion.assertResponse( testResponse ));
   }
   
   public void testAnyFragmentMatch() throws Exception
   {
		assertion.setContent( readResource( "/testFragment.xml") );
		assertion.setPath( "//company" );

		assertNotNull( assertion.assertResponse( testResponse ));
   }
   
   public void testLastElementTextMatch() throws Exception
	{
		assertion.setPath( "//company[last()]/companyName/text()" );
		assertion.setContent( "Bonnier Zoo Frlag AB" );
		
		assertNotNull( assertion.assertResponse( testResponse ));
	}
   
   public void testElementCountMatch() throws Exception
	{
		assertion.setPath( "count(//company)" );
		assertion.setContent( "20" );
		
		assertNotNull( assertion.assertResponse( testResponse ));
	}
   
   public void testAnyElementTextMatch() throws Exception
	{
		assertion.setPath( "declare namespace env='http://schemas.xmlsoap.org/soap/envelope/';" +
				"declare namespace urn='urn:schema:v1:companyservice:applications:bis.bonnier.se';" +
				"declare namespace urn1='urn:v1:companysearch:common:bis.bonnier.se';" +
               "/env:Envelope/env:Body/urn:searchResponse/urn1:searchResult/company/companyName/text()" );
		assertion.setContent( "Bonnier Otto Karl Adam" );
		
		assertNotNull( assertion.assertResponse( testResponse ));
	}

   public void testAnyElementTextFail() throws Exception
	{
		assertion.setPath( "declare namespace env='http://schemas.xmlsoap.org/soap/envelope/';" +
				"declare namespace urn='urn:schema:v1:companyservice:applications:bis.bonnier.se';" +
				"declare namespace urn1='urn:v1:companysearch:common:bis.bonnier.se';" +
               "/env:Envelope/env:Body/urn:searchResponse/urn1:searchResult/company/companyName/text()" );
		assertion.setContent( "Bonnier Otto Karl Adams" );
		
		try
		{
			assertNotNull( assertion.assertResponse(testResponse));
			assertFalse( "assertion should have failed", true );
		}
		catch (Exception e)
		{
		}
	}

	private String readResource(String string) throws Exception
	{
		BufferedReader reader = new BufferedReader( new InputStreamReader( getClass().getResourceAsStream( string ) ));
		StringBuffer result = new StringBuffer();
		
		String line = reader.readLine();
		while( line != null )
		{
			result.append( line );
			line = reader.readLine();
		}
		
		return result.toString();
	}

	private RequestAssertionConfig createConfig( String path, String content ) throws XmlException
	{
		return RequestAssertionConfig.Factory.parse( 
				"<con:configuration xmlns:con=\"http://eviware.com/soapui/config\">" +
				"<path>" + path + "</path><content>" + content + "</content></con:configuration>" );
	}
	
}
